#!/usr/bin/env python3
"""
https://www.wikidata.org/wiki/User:ProteinBoxBot/SPARQL_Examples

PREFIX wdt: <http://www.wikidata.org/prop/direct/>
PREFIX wd: <http://www.wikidata.org/entity/>
PREFIX bd: <http://www.bigdata.com/rdf#>
PREFIX up: <http://purl.uniprot.org/core/>
PREFIX uniprotkb:<http://purl.uniprot.org/uniprot/>
"""
###
import sys,os,logging
import pandas as pd
from wikidataintegrator.wdi_core import WDItemEngine

#############################################################################
def Rq2Df(rq):
  r = WDItemEngine.execute_sparql_query(rq)['results']['bindings']
  df = pd.DataFrame([{k:v['value'] for k,v in item.items()} for item in r])
  logging.debug(f"rows: {df.shape[0]}; cols: {df.shape[1]}")
  return(df)

#############################################################################
def ListDrugTargetPairs(fout=None):
  "List drugs with known targets."
  rq="""SELECT DISTINCT ?drug ?drugLabel ?gene_product ?gene_productLabel WHERE {
  ?drug wdt:P129 ?gene_product . 
  SERVICE wikibase:label { bd:serviceParam wikibase:language "en" . }
}"""
  df = Rq2Df(rq)
  if fout is not None: df.to_csv(fout, '\t', index=False)
  else: return df

#############################################################################
def ListGeneDiseasePairs(fout=None):
  "List genes with associated diseases."
  rq="""SELECT DISTINCT ?gene ?geneLabel ?disease ?diseaseLabel WHERE {
  ?gene wdt:P2293 ?disease . 
  SERVICE wikibase:label { bd:serviceParam wikibase:language "en" . }
}"""
  df = Rq2Df(rq)
  if fout is not None: df.to_csv(fout, '\t', index=False)
  else: return df

#############################################################################
def Test(fout=None):
  rq="""SELECT ?item ?itemLabel WHERE {
  ?item wdt:P279 wd:Q1049021 .
  SERVICE wikibase:label { bd:serviceParam wikibase:language "en" . }
}"""
  df = Rq2Df(rq)
  if fout is not None: df.to_csv(fout, '\t', index=False)
  else: return df

#############################################################################



#!/usr/bin/env python3
"""
Utility functions for PUG SOAP web services, including:
  - PubChem Identifier Exchange Service (PIES), https://pubchem.ncbi.nlm.nih.gov/idexchange/idexchange.cgi
  - PubChem Standardization Service, https://pubchem.ncbi.nlm.nih.gov/standardize/standardize.cgi
  - PubChem Structure Search, https://pubchem.ncbi.nlm.nih.gov/search/search.cgi
See Save Query button for template XML.
https://pubchemdocs.ncbi.nlm.nih.gov/pug-soap
https://pubchemdocs.ncbi.nlm.nih.gov/pug-soap-reference
https://pubchem.ncbi.nlm.nih.gov/pug_soap/pug_soap.cgi?wsdl
"""
import sys,os,io,re,requests,urllib.request,time,logging

from xml.etree import ElementTree

from ...util import xml

XMLHEADER = """\
<?xml version="1.0"?>
<!DOCTYPE PCT-Data PUBLIC "-//NCBI//NCBI PCTools/EN" "NCBI_PCTools.dtd">"""
#
SOAP_ACTIONS = {
	"standardize":	"http://pubchem.ncbi.nlm.nih.gov/Standardize",
	"idexchange":	"http://pubchem.ncbi.nlm.nih.gov/IDExchange",
	"search_exact":	"http://pubchem.ncbi.nlm.nih.gov/SubstructureSearch",
	"search_substructure":	"http://pubchem.ncbi.nlm.nih.gov/SubstructureSearch",
	"search_similarity":	"http://pubchem.ncbi.nlm.nih.gov/SimilaritySearch2D",
	}
#
API_HOST="pubchem.ncbi.nlm.nih.gov"
API_BASE_PATH="/pug/pug.cgi"
API_BASE_URL=f"https://{API_HOST}{API_BASE_PATH}"
POLL_WAIT=10
MAX_WAIT=300
SIM_CUTOFF=0.80
RETRY_NMAX=10;
#
#############################################################################
class PugSoapRequest:
  """
Normal status values: "success", "hit-limit", "queued", "running"
Error status values: "server-error"
  """
  def __init__(self, pugxml, base_url):
    self.base_url = base_url
    self.pugxml = pugxml
    self.reqid = None
    self.status = None
    self.url = None
    self.qkey = None
    self.wenv = None
    self.error = None
    self.ntries = None
    self.out_struct = None
    self.parsePugXml(pugxml)
    self.qxml_template="""\
{header}
<PCT-Data><PCT-Data_input>
  <PCT-InputData><PCT-InputData_request>
    <PCT-Request>
      <PCT-Request_reqid>{REQID}</PCT-Request_reqid>
      <PCT-Request_type value="{MODE}"/>
    </PCT-Request>
  </PCT-InputData_request></PCT-InputData>
</PCT-Data_input></PCT-Data>
"""

  def getStatus(self):
    logging.debug(f"Connecting {self.base_url}...")
    response = requests.post(self.base_url, data=(self.qxml_template.format(header=XMLHEADER, REQID=self.reqid, MODE="status")), headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":"http://pubchem.ncbi.nlm.nih.gov/GetOperationStatus"})
    self.parsePugXml(response.text)

  def cancel(self):
    logging.debug(f"Connecting {self.base_url}...")
    response = requests.post(self.base_url, data=(self.qxml_template.format(header=XMLHEADER, REQID=self.reqid, MODE="cancel")), headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":"http://pubchem.ncbi.nlm.nih.gov/GetOperationStatus"})
    self.parsePugXml(response.text)

  def parsePugXml(self, pugxml):
    logging.debug(pugxml)
    try:
      elt = ElementTree.parse(io.StringIO(pugxml))
    except Exception as e:
      logging.error(f"XML parse error: {e}; xml={pugxml}")
      sys.exit(1)
    statuss = xml.Utils.GetAttr(elt, 'PCT-Status', 'value')
    self.status = statuss[0] if statuss else None
    errors = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Status-Message_message')
    self.error = errors[0] if errors else None
    reqids = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Waiting_reqid')
    self.reqid = int(reqids[0]) if reqids else None
    urls = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Download-URL_url')
    self.url = urls[0] if urls else None
    qkeys = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Entrez_query-key')
    self.qkey = qkeys[0] if qkeys else None
    wenvs = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Entrez_webenv')
    self.wenv = wenvs[0] if wenvs else None
    #Standardize:
    out_structs = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Structure_structure_string')
    self.out_struct = out_structs[0].rstrip() if out_structs else None #smiles or inchi
    if self.out_struct is None:
      out_struct_lines = xml.Utils.GetLeafValsByTagName(elt, 'PCT-Structure_structure_strings_E')
      self.out_struct = ('\n'.join(out_struct_lines)) if out_struct_lines else None #sdf

  def waitForResult(self, poll_wait, max_wait):
    t0=time.time()
    while True:
      time.sleep(poll_wait)
      if time.time()-t0>max_wait:
        logging.error(f"Max wait exceeded ({max_wait} sec); quitting.")
        self.cancel()
      logging.info(f"Polling PUG [reqid={self.reqid}]...")
      self.getStatus()
      if self.status in ("success", "hit-limit"): 
        if self.url is not None:
          logging.info(f"{self.status}: Got url [reqid={self.reqid}]: {self.url}")
        elif self.qkey is not None:
          logging.info(f"{self.status}: Got qkey [reqid={self.reqid}]: {self.qkey}")
        elif self.out_struct is not None:
          logging.info(f"{self.status}: Got out_struct [reqid={self.reqid}]: {self.out_struct}")
        else:
          logging.error(f"Aaack! status={self.status} but no url, qkey, or out_struct??? [reqid={self.reqid}]")
        break
      logging.info(f"{time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))} elapsed; {poll_wait} sec wait; status={self.status}")
      if self.error:
        logging.info(self.error)
        if re.search('No result found', self.error, re.I):
          logging.info(f"No result found [reqid={self.reqid}].")
          break

#############################################################################
def SubmitRequest_StructuralSearch(query, searchtype, sim_cutoff, active=False,
nmax=10000, base_url=API_BASE_URL):
  qxml=f"""{XMLHEADER}
<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_query>
        <PCT-Query>
          <PCT-Query_type>
            <PCT-QueryType>
              <PCT-QueryType_css>
                <PCT-QueryCompoundCS>
                  <PCT-QueryCompoundCS_query>
                    <PCT-QueryCompoundCS_query_data>{query}</PCT-QueryCompoundCS_query_data>
                  </PCT-QueryCompoundCS_query>
                  <PCT-QueryCompoundCS_type>
"""
  if searchtype.lower()=='search_similarity':
    qxml+=f"""\
                    <PCT-QueryCompoundCS_type_similar>
                      <PCT-CSSimilarity>
                        <PCT-CSSimilarity_threshold>{int(sim_cutoff*100):d}</PCT-CSSimilarity_threshold>
                      </PCT-CSSimilarity>
                    </PCT-QueryCompoundCS_type_similar>
"""
  elif searchtype.lower()=='search_substructure':
    qxml+="""\
                    <PCT-QueryCompoundCS_type_subss>
                      <PCT-CSStructure>
                        <PCT-CSStructure_isotopes value="false"/>
                        <PCT-CSStructure_charges value="false"/>
                        <PCT-CSStructure_tautomers value="false"/>
                        <PCT-CSStructure_rings value="false"/>
                        <PCT-CSStructure_bonds value="true"/>
                        <PCT-CSStructure_chains value="true"/>
                        <PCT-CSStructure_hydrogen value="false"/>
                      </PCT-CSStructure>
                    </PCT-QueryCompoundCS_type_subss>
"""
  else: #search_exact
    qxml+="""\
                    <PCT-QueryCompoundCS_type_identical>
                      <PCT-CSIdentity value="same-stereo-isotope">5</PCT-CSIdentity>
                    </PCT-QueryCompoundCS_type_identical>
"""
  qxml+=f"""\
                  </PCT-QueryCompoundCS_type>
                  <PCT-QueryCompoundCS_results>{nmax}</PCT-QueryCompoundCS_results>
                </PCT-QueryCompoundCS>
              </PCT-QueryType_css></PCT-QueryType>
"""
  if active:
    qxml+="""\
            <PCT-QueryType>
              <PCT-QueryType_cel>
                <PCT-QueryCompoundEL>
                  <PCT-QueryCompoundEL_activity>
                    <PCT-CLByBioActivity value="active">2</PCT-CLByBioActivity>
                  </PCT-QueryCompoundEL_activity>
                </PCT-QueryCompoundEL>
              </PCT-QueryType_cel>
            </PCT-QueryType>
"""
  qxml+="""\
          </PCT-Query_type>
        </PCT-Query>
      </PCT-InputData_query>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>
"""
  logging.debug(f"qxml: {qxml}")
  logging.info(f"StructuralSearch: base_url: {base_url}; searchtype: {searchtype}{'(sim>='+str(sim_cutoff)+')' if searchtype.lower()=='search_similarity' else '' }; query: {query}")
  response = requests.post(base_url, data=qxml, headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":SOAP_ACTIONS[searchtype]})
  return PugSoapRequest(response.text, base_url)

#############################################################################
def SubmitRequest_IDExchange(ids, ifmt, ofmt, operator, do_gz, base_url=API_BASE_URL):
  # Input formats: cid, sid, smiles, inchi, inchikey
  # Output formats: cid, sid, smiles, inchi, inchikey
  # Operators: same (same cid), parent (parent cid), samepar (same parent compound)
  fmt2tag = {"cid":"cid", "sid":"sid", "smiles":"smiles", "inchi":"inchis", "inchikey":"inchikeys"}
  qxml=(f"""{XMLHEADER}
<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_query>
        <PCT-Query>
          <PCT-Query_type>
            <PCT-QueryType>
              <PCT-QueryType_id-exchange>
                <PCT-QueryIDExchange>
                  <PCT-QueryIDExchange_input>
                    <PCT-QueryUids>
""")
  if ifmt in ("cid", "sid"):
    qxml+=(f"""\
                      <PCT-QueryUids_ids>
                        <PCT-ID-List>
                          <PCT-ID-List_db>{"pccompound" if ifmt=="cid" else "pcsubstance"}</PCT-ID-List_db>
                          <PCT-ID-List_uids>
""")
    for id_this in ids:
      qxml+=(f"""\
                            <PCT-ID-List_uids_E>{id_this}</PCT-ID-List_uids_E>
""")
    qxml+=(f"""\
                          </PCT-ID-List_uids>
                        </PCT-ID-List>
                      </PCT-QueryUids_ids>
""")
  else:
    qxml+=(f"""\
                      <PCT-QueryUids_{fmt2tag[ifmt]}>
""")
    for id_this in ids:
      qxml+=(f"""\
                        <PCT-QueryUids_{fmt2tag[ifmt]}_E>{id_this}</PCT-QueryUids_{fmt2tag[ifmt]}_E>
""")
    qxml+=(f"""\
                      </PCT-QueryUids_{fmt2tag[ifmt]}>
""")
  qxml+=(f"""\
                    </PCT-QueryUids>
                  </PCT-QueryIDExchange_input>
                  <PCT-QueryIDExchange_operation-type value="{operator}"/>
                  <PCT-QueryIDExchange_output-type value="{ofmt}"/>
                  <PCT-QueryIDExchange_output-method value="file-pair"/>
                  <PCT-QueryIDExchange_compression value="{'gzip' if do_gz else 'none'}"/>
                </PCT-QueryIDExchange>
              </PCT-QueryType_id-exchange>
            </PCT-QueryType>
          </PCT-Query_type>
        </PCT-Query>
      </PCT-InputData_query>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>
""")
  logging.debug(f"qxml: {qxml}")
  logging.debug(f"IDExchange: base_url: {base_url}: IDs: {len(ids)}")
  response = requests.post(base_url, data=qxml, headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":SOAP_ACTIONS["idexchange"]})
  if response.status_code!=200:
    logging.error(f"(status_code={response.status_code})")
  logging.debug(f"(response.text={response.text})")
  return PugSoapRequest(response.text, base_url)

#############################################################################
def SubmitRequest_Standardize(id_this, ifmt, ofmt, do_gz, base_url=API_BASE_URL):
  """Only one per request allowed."""
  qxml=(f"""{XMLHEADER}
<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_standardize>
        <PCT-Standardize>
          <PCT-Standardize_structure>
            <PCT-Structure>
              <PCT-Structure_structure>
                <PCT-Structure_structure_string>{id_this}</PCT-Structure_structure_string>
              </PCT-Structure_structure>
              <PCT-Structure_format>
                <PCT-StructureFormat value="{ifmt}"/>
              </PCT-Structure_format>
            </PCT-Structure>
          </PCT-Standardize_structure>
          <PCT-Standardize_oformat>
            <PCT-StructureFormat value="{ofmt}"/>
          </PCT-Standardize_oformat>
        </PCT-Standardize>
      </PCT-InputData_standardize>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>
""")
  logging.debug(f"qxml: {qxml}")
  logging.debug(f"Standardize: base_url: {base_url}")
  response = requests.post(base_url, data=qxml, headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":SOAP_ACTIONS["standardize"]})
  return PugSoapRequest(response.text, base_url)

#############################################################################
def MonitorQuery(qkey, webenv, fmt='smiles', do_gz=False, base_url=API_BASE_URL):
  qxml=(f"""{XMLHEADER}
<PCT-Data><PCT-Data_input><PCT-InputData>
  <PCT-InputData_download><PCT-Download> 
    <PCT-Download_uids><PCT-QueryUids>
      <PCT-QueryUids_entrez><PCT-Entrez>
        <PCT-Entrez_db>pccompound</PCT-Entrez_db>
        <PCT-Entrez_query-key>{qkey}</PCT-Entrez_query-key>
        <PCT-Entrez_webenv>{webenv}</PCT-Entrez_webenv>
      </PCT-Entrez></PCT-QueryUids_entrez>
    </PCT-QueryUids></PCT-Download_uids>
    <PCT-Download_format value="{fmt}"/>
    <PCT-Download_compression value="{'gzip' if do_gz else 'none'}"/>
  </PCT-Download></PCT-InputData_download>
</PCT-InputData></PCT-Data_input></PCT-Data>
""")
  logging.debug(f"qxml: {qxml}")
  logging.debug(f"[MonitorQuery] Requesting: base_url: {base_url}; qkey: {qkey}")
  response = requests.post(base_url, data=qxml, headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":"http://pubchem.ncbi.nlm.nih.gov/Download"})
  return PugSoapRequest(response.text, base_url)

#############################################################################
def DownloadResults(download_url, ofmt, do_gz=False, fout=sys.stdout):
  """FTP urls not handled by requests package?!"""

#  response = requests.get(download_url, headers={"Accept":"text/soap+xml; charset=utf-8", "SOAPAction":"http://pubchem.ncbi.nlm.nih.gov/Download"})
#  if response.status_code!=200:
#    logging.error(f"[DownloadResults] (status_code={response.status_code})")
#  if type(response.content) is bytes:
#    fout.write(response.content)
#  else:
#    fout.write(response.content.encode("utf-8"))
#  logging.info(f"Data downloaded to {fout.name}")

  request = urllib.request.Request(url=download_url)
  logging.debug(f"request type = {request.type}")
  logging.debug(f"request host = {request.host}")
  logging.debug(f"request method = {request.get_method()}")
  for i_try in range(RETRY_NMAX):
    try:
      with urllib.request.urlopen(request, timeout=POLL_WAIT) as response:
        fbytes = response.read()
        fout.write(fbytes if do_gz else fbytes.decode("utf-8"))
    except Exception as e:
      logging.warn(f"RETRY({i_try+1}/{RETRY_NMAX}): {e}")
      if i_try<RETRY_NMAX:
        time.sleep(POLL_WAIT)
        continue
    break

#############################################################################



#!/usr/bin/env python3
"""
See https://github.com/chembl/chembl_webresource_client
New with ChEMBL 25 March 2019:
https://chembl.gitbook.io/chembl-interface-documentation/web-services/chembl-data-web-services
"""
###
import sys,os,argparse,logging
import csv

from chembl_webresource_client.new_client import new_client

NCHUNK = 50

#############################################################################
act_tags_selected = ['activity_id', 'assay_chembl_id', 'assay_type', 'src_id',
	'relation', 'standard_relation',
	'target_chembl_id', 'target_pref_name', 'target_organism', 'target_tax_id',
	'molecule_chembl_id', 'parent_molecule_chembl_id', 'molecule_pref_name',
	'canonical_smiles',
	'document_chembl_id', 'document_year',
	'pchembl_value', 'value', 'standard_value',
	'text_value', 'published_value',
	'units', 'qudt_units', 'uo_units', 'published_units', 'standard_units',
	'type', 'published_type', 'standard_type']
def CID2Activity(cids, args, fout):
  """
Compounds to targets through activities.
Normally select biochemical assays, protein targets, activities with pChembl.
Process in chunks to avoid timeouts due to size.
  """
  tags=None;
  n_act=0;
  i_start=(args.skip if args.skip else 0)
  i_end = min(args.nmax, len(cids)-i_start) if args.nmax else (len(cids)-i_start)
  writer = csv.writer(fout, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
  for i in range(i_start, i_end, NCHUNK):
    if args.include_phenotypic:
      acts = new_client.activity.filter(molecule_chembl_id__in=cids[i:i+NCHUNK]).only(act_tags_selected)
    else:
      acts = new_client.activity.filter(molecule_chembl_id__in=cids[i:i+NCHUNK], pchembl_value__isnull=False).only(act_tags_selected)
    for act in acts:
      if not tags:
        tags = list(act.keys())
        writer.writerow(tags)
      writer.writerow([(act[tag] if tag in act else '') for tag in tags])
      n_act+=1
    logging.info('Progress: CIDs: %d / %d ; activities: %d'%(i-i_start, i_end-i_start, n_act))
  logging.info('Output activities: %d'%n_act)

#############################################################################
def TID2Targetcomponents(tids, args, fout):
  t_tags=['target_chembl_id','target_type','organism', 'species_group_flag', 'tax_id']
  tc_tags=['component_id','component_type','component_description','relationship', 'accession']
  ntc=0;
  writer = csv.writer(fout, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
  for i in range(0, len(tids), NCHUNK):
    targets = new_client.target.filter(target_chembl_id__in=tids[i:i+NCHUNK])
    for t in targets:
      logging.debug('target tags: %s'%(str(t.keys())))
      logging.debug('target: %s'%(str(t)))
      t_vals=[(t[tag] if tag in t else '') for tag in t_tags]
      for tc in t['target_components']:
        if tc['component_type'] != 'PROTEIN':
          continue
        if ntc==0:
          writer.writerow(t_tags+tc_tags)
        tc_vals=[(tc[tag] if tag in tc else '') for tag in tc_tags]
        writer.writerow(t_vals+tc_vals)
        ntc+=1
  logging.info('Output target components (PROTEIN): %d'%ntc)

#############################################################################
def DID2Documents(dids, args, fout):
  d_tags=['document_chembl_id', 'doc_type', 'src_id', 'pubmed_id',
	'patent_id', 'doi', 'doi_chembl', 'year', 'journal', 'authors',
	'volume', 'issue', 'title', 'journal_full_title', 'abstract']
  ndoc=0;
  writer = csv.writer(fout, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
  for i in range(0, len(dids), NCHUNK):
    documents = new_client.document.filter(document_chembl_id__in=dids[i:i+NCHUNK])
    for d in documents:
      logging.debug('document tags: %s'%(str(d.keys())))
      logging.debug('document: %s'%(str(d)))
      d_vals=[(d[tag] if tag in d else '') for tag in d_tags]
      if ndoc==0:
        writer.writerow(d_tags)
      writer.writerow(d_vals)
      ndoc+=1
  logging.info('Output documents: %d'%ndoc)

#############################################################################
def InchiKey2Molecule(inkeys, args, fout):
  m_tags=[
	'availability_type', 'biotherapeutic', 'black_box_warning', 'chebi_par_id', 
	'chirality', 'dosed_ingredient', 'first_approval', 'first_in_class', 
	'helm_notation', 'indication_class', 'inorganic_flag', 'max_phase', 
	'molecule_chembl_id', 'molecule_type', 'natural_product', 'oral', 
	'parenteral', 'polymer_flag', 'pref_name', 'prodrug', 'structure_type', 
	'therapeutic_flag', 'topical', 'usan_stem', 'usan_stem_definition', 
	'usan_substem', 'usan_year', 'withdrawn_class', 'withdrawn_country', 
	'withdrawn_flag', 'withdrawn_reason', 'withdrawn_year']
  n_mol=0;
  writer = csv.writer(fout, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
  for i in range(0, len(inkeys), NCHUNK):
    mol = new_client.molecule
    mols = mol.get(inkeys[i:i+NCHUNK])
    for ii,m in enumerate(mols):
      inkey = inkeys[i+ii]
      logging.debug('mol tags: %s'%(str(m.keys())))
      logging.debug('mol: %s'%(str(m)))
      m_vals=[inkey]+[(m[tag] if tag in m else '') for tag in m_tags]
      if n_mol==0:
        writer.writerow(['inchikey']+m_tags)
      writer.writerow(m_vals)
      n_mol+=1
  logging.info('Output mols: %d'%n_mol)

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='ChEMBL REST API client: lookup by IDs')
  ops = ['cid2Activity', 'tid2Targetcomponents', 'did2Documents', 'inchikey2Mol']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input file, IDs")
  parser.add_argument("--ids", help="input IDs, comma-separated")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--skip", type=int)
  parser.add_argument("--nmax", type=int)
  parser.add_argument("--include_phenotypic", action="store_true", help="else pChembl required")
  parser.add_argument("-v","--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  if not (args.ifile or args.id):
    parser.error('--i or --id required.')
  if args.ofile:
    fout = open(args.ofile, 'w')
  else:
    fout = sys.stdout

  ids=[]
  if args.ifile:
    with open(args.ifile, 'r') as fin:
      reader = csv.reader(fin, delimiter='\t')
      for row in reader:
        ids.append(row[0])
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())
  logging.info('Input IDs: %d'%len(ids))

  if args.op == 'cid2Activity':
    CID2Activity(ids, args, fout)
 
  elif args.op == 'inchikey2Mol':
    InchiKey2Molecule(ids, args, fout)
 
  elif args.op == 'tid2Targetcomponents':
    TID2Targetcomponents(ids, args, fout)
 
  elif args.op == 'did2Documents':
    DID2Documents(ids, args, fout)
 



#!/usr/bin/env python3
"""
OpenTargets REST API client, using the Python client package.
http://opentargets.readthedocs.io/
"""
import sys,os,re,time,argparse,json,csv,logging

from opentargets import OpenTargetsClient

from .. import opentargets #BioClients module.

#############################################################################
if __name__=='__main__':
  IDTYPES = ["gene", "disease"]
  #SOURCES = ["cancer_gene_census", "chembl", "crispr", "europepmc", "eva", "eva_somatic", "expression_atlas", "gene2phenotype", "genomics_england", "gwas_catalog", "intogen", "ot_genetics_portal", "phenodigm", "phewas_catalog", "postgap", "progeny", "reactome", "slapenrich", "sysbio", "uniprot", "uniprot_literature", "uniprot_somatic"]
  epilog = """Examples:
Gene IDs: ENSG00000163914, ENSG00000072110;
Disease IDs: EFO_0002630, EFO_1001473, HP_0011446, HP_0100543, EFO_0002508 ;
Search terms: prostate, alzheimer, lymphoma
"""
  parser = argparse.ArgumentParser(description='OpenTargets REST API client utility', epilog=epilog)
  ops = ['searchAssociations', 'getEvidence']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input file, gene or disease IDs")
  parser.add_argument("--ids", help="(1) target ID, gene symbol or ENSEMBL; (2) disease ID, EFO_ID; or (3) search term (comma-separated)")
  parser.add_argument("--idtype", choices=IDTYPES, default="disease")
  #parser.add_argument("--source", choices=SOURCES)
  parser.add_argument("--minscore", type=float, help="minimum overall association score")
  parser.add_argument("--nmax", type=int, default=0, help="max results")
  parser.add_argument("--skip", type=int, default=0, help="skip results")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  t0 = time.time()

  fout = open(args.ofile,"w+") if args.ofile else sys.stdout

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      if line.rstrip(): ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids)
  if ids: logging.info('Input IDs: {}'.format(len(ids)))

  otclient = OpenTargetsClient()

  logging.debug(otclient.get_stats().info)

  if args.op=='searchAssociations':
    if not ids: parser.error('--i or --ids required.')
    opentargets.SearchAssociations(otclient, ids, args.idtype, args.minscore, args.skip, args.nmax, fout)

  elif args.op=='getEvidence':
    parser.error('Unimplemented operation: {}'.format(args.op))
    if not ids: parser.error('--i or --ids required.')
    #opentargets.Target2Disease_Evidence(otclient, tid, args.did, fout)

  else:
    parser.error('Invalid operation: {}'.format(args.op))

  logging.info("Elapsed: {}".format(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))




#!/usr/bin/env python3
"""
Utility for PDB REST API.
See: http://www.rcsb.org/pdb/software/rest.do
"""
###
import sys,os,re,json,argparse,time,logging
#
from .. import pdb
#
API_HOST='www.rcsb.org'
API_BASE_PATH='/pdb/rest'
#
##############################################################################
if __name__=='__main__':
  epilog="""
Example keywords: ACTIN, Example UniProts: P50225'.
get_uniprots functionality may be discontinued by PDB.
"""
  parser = argparse.ArgumentParser(description='PDB REST API client', epilog=epilog)
  ops = ['show_counts',
    'list_proteins', 'list_ligands',
    'search',
    'get_proteins', 'get_ligands', 'get_ligands_LID2SDF',
    'get_uniprots']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--ids", dest="ids", help="PDB IDs, comma-separated")
  parser.add_argument("--i", dest="ifile", help="input file, PDB IDs")
  parser.add_argument("--druglike", action="store_true", help="druglike ligands only (organic; !polymer; !monoatomic)")
  parser.add_argument("--o", dest="ofile", help="output file (TSV)")
  parser.add_argument("--qstr", help="search query")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)

  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL='https://'+args.api_host+args.api_base_path

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())

  if args.ofile:
    fout = open(args.ofile, "w+")
  else:
    fout = sys.stdout

  t0=time.time()

  if args.op == "get_proteins":
    if not ids: parser.error('ID[s] required.')
    pdb.GetProteins(BASE_URL, ids, fout)

  elif args.op == "get_uniprots":
    if not ids: parser.error('ID[s] required.')
    pdb.GetUniprots(BASE_URL, ids, fout)

  elif args.op == "get_ligands":
    if not ids: parser.error('ID[s] required.')
    pdb.GetLigands(BASE_URL, ids, args.druglike, fout)

  elif args.op == "get_ligands_LID2SDF":
    if not ids: parser.error('ID[s] required.')
    pdb.GetLigands_LID2SDF(BASE_URL, ids, fout)

  elif args.op == "list_proteins":
    pdb.ListProteins(BASE_URL, fout)

  elif args.op == "list_ligands":
    pdb.ListLigands(BASE_URL, args.druglike, fout)

  elif args.op == "show_counts":
    pdb.ShowCounts(BASE_URL)

  elif args.op == "search":
    ids = pdb.SearchByKeywords(BASE_URL, args.qstr)
    logging.info('protein count: %d'%(len(ids)))
    pdb.GetProteins(BASE_URL, ids, fout)

  else:
    parser.error('Invalid operation: %s'%args.op)

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))))



#!/usr/bin/env python
#############################################################################
### pubchem_actives.py - extract SIDs from PubChem CSV assay file[s]
### activity outcomes: 
###   1 = inactive
###   2 = active
###   3 = inconclusive
###   4 = unspecified
###   multiple, differing 1, 2 or 3 = discrepant
###   not 4 = tested
###
### If a single assay is under consideration, compounds are separated into
### all five defined categories.
###
### If multiple assays are under consideration, a compound active in ANY
### assay is considered active by this program, and all other compounds
### are considered inactive.
#############################################################################
###  WARNING: This program is a RAM HOG!  3GB when used for ~900 assays
###  and ~400k SIDs.
#############################################################################
###  July 2010: The format of the downloaded csv files has changed.  Today
###  I count 189947 .csv.gz files in bioassay/csv/data/ subdirs.
###  Running the program with
### entrez_assay_search.pl \
###         -v \
###         -mlpcn \
###         -out_aids data/pubchem_mlpcn.aid
#### (1998 assays)
### pubchem_actives.py \
###         --aidfile data/pubchem_mlpcn.aid \
###         --out_all data/pubchem_mlpcn.cid  
### 
### still working after >1week.
#############################################################################
###  to do:
###    [x] use tmp db (dbm? gdbm?) instead of huge Python containers.
###        dbm seems to have some problems.
###        REPLACED sids hash with sids_db database.
###        Use sids_list for iteration.
###        Don't use /tmp due to size (5+GB).
###
###    [x] allow jobs to be resumed using gdbm file
###    [ ] output compound ids with activity stats on one line
###    [ ] fix  --out_all_stats
#############################################################################
import os,sys,re,getopt,gzip,tempfile
try:
  import gdbm
except:
  import dbm as gdbm
from ... import pubchem

PROG=os.path.basename(sys.argv[0])
DATADIR="/home/data/pubchem/bioassay/csv/data"

#############################################################################
if __name__=='__main__':
  usage='''
  %(PROG)s - extract SIDs from assay CSV files

  options:
  --indir=<DIR>   ... dir w/ csv.gz assay data [%(DATADIR)s]
  or
  --infile=<FILE>   ... input csv.gz assay data

  --aids=<AIDS> ... list of AIDs to select within --indir dir
  --aidfile=<AIDFILE> ... AIDs to select within --indir dir
  --sidfile=<SIDFILE> ... SIDs to select (ignore others)

  --out_active=<outfile> ... active SIDs
  --out_inactive=<outfile> ... inactive SIDs
  --out_all=<outfile> ... all SIDs

  --out_all_stats=<outfile> ... all SIDs, w/ inac,act,incon,unsp

  only valid for single assay:
  --out_inconclusive=<outfile> ... output file of inconclusive SIDs
  --out_unspecified=<outfile> ... output file of unspecified SIDs
  --out_tested=<outfile> ... output file of not unspecified SIDs
  --out_discrepant=<outfile> ... output file of discrepant SIDs

  --inc_aids     ... append <tab>N_AIDS<tab>AIDs to SIDs
  --use_cids     ... use CIDs instead of SIDs (all i/o)
  --resume_gdbm_file=<gdbm_file> ... resume job with existing gdbm file
  --v            ... verbose
  --vv           ... very verbose
  --h            ... this help
'''%{'PROG':PROG,'DATADIR':DATADIR}

  def ErrorExit(msg):
    print >>sys.stderr,msg
    sys.exit(1)

  indir=DATADIR;
  infile=None; verbose=0;
  out_active_file=None; out_inactive_file=None;
  out_inconclusive_file=None; out_unspecified_file=None;
  out_tested_file=None; out_discrepant_file=None; out_all_file=None;
  out_all_stats_file=None;
  inc_aids=False; aids=None; aidfile=None; sidfile=None;
  use_cids=False; resume_gdbm_file=None;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','indir=',
  'aids=','aidfile=','infile=','inc_aids',
  'out_active=','out_inactive=',
  'out_inconclusive=','out_unspecified=','out_tested=',
  'out_discrepant=','out_all=','out_all_stats=','resume_gdbm_file=',
  'sidfile=','use_cids'])
  if not opts: ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': ErrorExit(usage)
    elif opt=='--indir': indir=val
    elif opt=='--infile': infile=val
    elif opt=='--out_active': out_active_file=val
    elif opt=='--out_inactive': out_inactive_file=val
    elif opt=='--out_inconclusive': out_inconclusive_file=val
    elif opt=='--out_unspecified': out_unspecified_file=val
    elif opt=='--out_discrepant': out_discrepant_file=val
    elif opt=='--out_tested': out_tested_file=val
    elif opt=='--out_all': out_all_file=val
    elif opt=='--out_all_stats': out_all_stats_file=val
    elif opt=='--inc_aids': inc_aids=True
    elif opt=='--aids': aids=val
    elif opt=='--aidfile': aidfile=val
    elif opt=='--sidfile': sidfile=val
    elif opt=='--use_cids': use_cids=True
    elif opt=='--resume_gdbm_file': resume_gdbm_file=val
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: ErrorExit('Illegal option: %s'%val)

  if not (indir or infile):
    ErrorExit('input dir or file required\n'+usage)

  fout_active=None
  if out_active_file:
    fout_active=open(out_active_file,'w')
    if not fout_active:
      ErrorExit('cannot open: %s'%out_active_file)
  fout_inactive=None
  if out_inactive_file:
    fout_inactive=open(out_inactive_file,'w')
    if not fout_inactive:
      ErrorExit('cannot open: %s'%out_inactive_file)

  aidset=[]
  if aids:
    for val in re.split('[\s,]+',aids):
      if not val.strip(): continue
      aidset.append(int(val.strip()))
  if aidfile:
    f=open(aidfile)
    if not f:
      ErrorExit('cannot open: %s'%aidfile)
    lines=f.readlines()
    for line in lines:
      if not line.strip(): continue
      aidset.append(int(line.strip()))
  sidset={}	## use a hash for speed (could be many sids)
  if sidfile:
    f=open(sidfile)
    if not f:
      ErrorExit('cannot open: %s'%sidfile)
    lines=f.readlines()
    for line in lines:
      if not line.strip(): continue
      line=re.sub('\s.*$','',line)
      sidset[int(line.strip())]=True
  if len(sidset.keys())==0: sidset=None

  files=[]
  if infile:
    try:
      aid=int(re.sub('\.csv\.gz','',os.path.basename(infile)))
    except:
      ErrorExit('cannot parse AID: "%s"'%infile)
    files.append((infile,aid))
  else:
    for root,dirs,fs in os.walk(indir):
      for dir in dirs:
        for f in os.listdir(os.path.join(root,dir)):
          if not re.search('\.csv\.gz',f): continue
          try:
            aid=int(re.sub('\.csv\.gz','',f))
          except:
            print >>sys.stderr, 'cannot parse AID: "%s"'%f
            continue
          if aidset and (aid not in aidset): continue
          fpath=os.path.join(root,dir,f)
          files.append((fpath,aid))

  if aidset:
    print >>sys.stderr, ('%d / %d files found (%.1f%%)'%
	(len(files),len(aidset),(100.0*len(files)/len(aidset))))

  for fpath,aid in files: #DEBUG
    print >>sys.stderr, aid, fpath #DEBUG
  sys.exit() #DEBUG

  if len(files)>1 and (out_inconclusive_file or out_unspecified_file or out_discrepant_file or out_tested_file):
    ErrorExit('--out_inconclusive/out_unspecified/out_discrepant/out_tested only valid for single assay.')
  fout_inconclusive=None
  if out_inconclusive_file:
    fout_inconclusive=open(out_inconclusive_file,'w')
    if not fout_inconclusive:
      ErrorExit('cannot open: %s'%out_inconclusive_file)
  fout_unspecified=None
  if out_unspecified_file:
    fout_unspecified=open(out_unspecified_file,'w')
    if not fout_unspecified:
      ErrorExit('cannot open: %s'%out_unspecified_file)
  fout_discrepant=None
  if out_discrepant_file:
    fout_discrepant=open(out_discrepant_file,'w')
    if not fout_discrepant:
      ErrorExit('cannot open: %s'%out_discrepant_file)
  fout_tested=None
  if out_tested_file:
    fout_tested=open(out_tested_file,'w')
    if not fout_tested:
      ErrorExit('cannot open: %s'%out_tested_file)
  fout_all=None
  if out_all_file:
    fout_all=open(out_all_file,'w')
    if not fout_all:
      ErrorExit('cannot open: %s'%out_all_file)
  fout_all_stats=None
  if out_all_stats_file:
    fout_all_stats=open(out_all_stats_file,'w')
    if not fout_all_stats:
      ErrorExit('cannot open: %s'%out_all_stats_file)

  if resume_gdbm_file:
    sids_db_path=resume_gdbm_file
    sids_db=gdbm.open(sids_db_path,'w')
  else:
    tempfile.tempdir=os.environ['HOME']+'/Downloads'
    fd,sids_db_path = tempfile.mkstemp('_sids_db',PROG)
    os.close(fd)
    sids_db=gdbm.open(sids_db_path,'n')
  sids_list=[]

  sids_active=[]; sids_inactive=[]; sids_inconclusive=[]; sids_unspecified=[];
  sids_discrepant=[]; sids_tested=[];
  n_datapoints_total=0
  i_file=0
  for fpath,aid in files:
    i_file+=1
    print >>sys.stderr, '%d. [%d]: %s'%(i_file,aid,fpath)
    if sids_db.has_key('AID_DONE_FLAG_%d'%aid):
      continue
    try:
      f=gzip.open(fpath)
    except:
      print >>sys.stderr, 'ERROR: could not open %s'%(fpath)
      continue
    ftxt=f.read()
    sids_this=pubchem.ftp.Utils.ExtractOutcomes(ftxt,sidset,use_cids)
    n_active=0; n_inactive=0; n_inconclusive=0; n_unspecified=0; n_discrepant=0;
    for sid in sids_this.keys():

      if not sids_db.has_key('%d'%sid):
        sids_db['%d'%sid]=''
        sids_list.append(sid)

      outcome=sids_this[sid]['outcome']

      sids_db['%d_%d'%(sid,aid)]=('%d'%outcome)
      aids_this=pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)])
      if aid not in aids_this:
        aids_this.append(aid)
      sids_db['%d'%(sid)]=pubchem.ftp.Utils.Ints2Str(aids_this)

      n_datapoints_total+=1
      if outcome==2: n_active+=1
      elif outcome==1: n_inactive+=1
      elif outcome==3: n_inconclusive+=1
      elif outcome==4: n_unspecified+=1
      elif outcome==5: n_discrepant+=1
      else: print >>sys.stderr, 'ERROR: outcome=%d'%(sids_this[sid]['outcome'])

    f.close()
    if verbose>1:
      print >>sys.stderr, '\t      active: %3d'%(n_active)
      print >>sys.stderr, '\t    inactive: %3d'%(n_inactive)
      print >>sys.stderr, '\tinconclusive: %3d'%(n_inconclusive)
      print >>sys.stderr, '\t unspecified: %3d'%(n_unspecified)
      print >>sys.stderr, '\t  discrepant: %3d'%(n_discrepant)
      print >>sys.stderr, '\t       total: %3d'%(len(sids_this.keys()))

    sids_db['AID_DONE_FLAG_%d'%aid]='done'	#file done; for resume

  for sid in sids_list:
    sid_is_active=False

    for aid in pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)]):
      if sids_db['%d_%d'%(sid,aid)]=='2':
        sids_active.append(sid)
        sid_is_active=True
        break

    if not sid_is_active:
      if len(files)==1:
        if sids_this[sid]['outcome']==1:
          sids_inactive.append(sid)
        elif sids_this[sid]['outcome']==3:
          sids_inconclusive.append(sid)
        elif sids_this[sid]['outcome']==4:
          sids_unspecified.append(sid)
        elif sids_this[sid]['outcome']==5:
          sids_discrepant.append(sid)
      else:
        sids_inactive.append(sid)

  if fout_active:
    f=fout_active
    if verbose:
      print >>sys.stderr, 'sorting %d active SIDs...'%(len(sids_active))
    sids_active.sort()
    for sid in sids_active:
      f.write('%d'%sid)
      if inc_aids:
        aids=[]

        for aid in pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)]):
          if sids_db['%d_%d'%(sid,aid)]=='2': aids.append(aid)

        f.write('\t%d\t%s'%(len(aids),pubchem.ftp.Utils.Ints2Str(aids)))
      f.write('\n')
    f.close()

  if fout_inactive:
    f=fout_inactive
    if verbose:
      print >>sys.stderr, 'sorting %d inactive SIDs...'%(len(sids_inactive))
    sids_inactive.sort()
    for sid in sids_inactive:
      f.write('%d'%sid)
      if inc_aids:
        aids=[]

        for aid in pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)]):
          if sids_db['%d_%d'%(sid,aid)]=='1': aids.append(aid)

        f.write('\t%d\t%s'%(len(aids),pubchem.ftp.Utils.Ints2Str(aids)))
      f.write('\n')
    f.close()

  if fout_inconclusive:
    f=fout_inconclusive
    if verbose:
      print >>sys.stderr, 'sorting %d inconclusive SIDs...'%(len(sids_inconclusive))
    sids_inconclusive.sort()
    for sid in sids_inconclusive:
      f.write('%d'%sid)
      f.write('\n')
    f.close()

  if fout_unspecified:
    f=fout_unspecified
    if verbose:
      print >>sys.stderr, 'sorting %d unspecified SIDs...'%(len(sids_unspecified))
    sids_unspecified.sort()
    for sid in sids_unspecified:
      f.write('%d'%sid)
      f.write('\n')
    f.close()

  if fout_discrepant:
    f=fout_discrepant
    if verbose:
      print >>sys.stderr, 'sorting %d discrepant SIDs...'%(len(sids_discrepant))
    sids_discrepant.sort()
    for sid in sids_discrepant:
      f.write('%d'%sid)
      f.write('\n')
    f.close()

  if fout_tested:
    f=fout_tested
    if verbose:
      print >>sys.stderr, 'sorting %d tested SIDs...'%(len(sids_tested))
    sids_tested=sids_active+sids_inactive+sids_inconclusive+sids_discrepant
    sids_tested.sort()
    for sid in sids_tested:
      f.write('%d'%sid)
      f.write('\n')
    f.close()

  if fout_all:
    f=fout_all
    sids_all=sids_active+sids_inactive+sids_inconclusive+sids_unspecified+sids_discrepant
    if verbose:
      print >>sys.stderr, 'sorting %d all SIDs...'%(len(sids_all))
    sids_all.sort()
    for sid in sids_all:
      f.write('%d'%sid)
      if inc_aids:

        aids=pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)])

        f.write('\t%d\t%s'%(len(aids),pubchem.ftp.Utils.Ints2Str(aids)))
      f.write('\n')
    f.close()

  if fout_all_stats:
    f=fout_all_stats
    sids_all=sids_active+sids_inactive+sids_inconclusive+sids_unspecified+sids_discrepant
    if verbose:
      print >>sys.stderr, 'sorting %d all SIDs...'%(len(sids_all))
    sids_all.sort()
    f.write('id,n_inactive,n_active,n_inconclusive,n_unspecified\n')
    for sid in sids_all:
      nact,nina,ninc,nuns = 0,0,0,0
      for aid in pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)]):
        if sids_db['%d_%d'%(sid,aid)]=='1': nina+=1
        elif sids_db['%d_%d'%(sid,aid)]=='2': nact+=1
        elif sids_db['%d_%d'%(sid,aid)]=='3': ninc+=1
        elif sids_db['%d_%d'%(sid,aid)]=='4': nuns+=1
      f.write('%d,%d,%d,%d,%d\n'%(sid,nina,nact,ninc,nuns))
    f.close()

  os.unlink(sids_db_path)
  print >>sys.stderr, 'DEBUG: gdbm file: %s'%(sids_db_path)

  id='SID'
  if use_cids: id='CID'
  print >>sys.stderr, '%s: number of assay files: %d'%(PROG,len(files))
  print >>sys.stderr, '%s: active %ss: %d'%(PROG,id,len(sids_active))
  print >>sys.stderr, '%s: inactive %ss: %d'%(PROG,id,len(sids_inactive))
  if len(files)==1:
    print >>sys.stderr, '%s: inconclusive %ss: %d'%(PROG,id,len(sids_inconclusive))
    print >>sys.stderr, '%s: unspecified %ss: %d'%(PROG,id,len(sids_unspecified))
    print >>sys.stderr, '%s: discrepant %ss: %d'%(PROG,id,len(sids_discrepant))
    print >>sys.stderr, '%s: total %ss: %d'%(PROG,id,len(sids_active)+len(sids_inactive)+len(sids_inconclusive)+len(sids_unspecified)+len(sids_discrepant))
  else:
    print >>sys.stderr, '%s: total %ss: %d'%(PROG,id,len(sids_active)+len(sids_inactive))
  print >>sys.stderr, '%s: total datapoints: %d'%(PROG,n_datapoints_total)




#!/usr/bin/env python3
"""
Client for SLAP REST API.

See: http://slapfordrugtargetprediction.wikispaces.com/API
"""
###
import sys,os,re,argparse,time,logging
#
from ... import chem2bio2rdf as c2b2r
#
##############################################################################
if __name__=='__main__':
  API_HOST='cheminfov.informatics.indiana.edu'
  API_BASE_PATH='/rest/Chem2Bio2RDF/slap'
  epilog="""\
D2T: drug-to-target associations;
T2D: target-to-drug associations;
DTP: drug-target paths, subnet;
DSS: drug-to-bio-similar-drugs search;
DDS: drug-drug similarity;
Examples: CIDs: 5591 TIDs: PPARG
"""
  parser = argparse.ArgumentParser(description="SLAP REST API client)", epilog=epilog)
  ops = [ "D2T", "T2D", "DTP", "DSS", "DDS" ]
  parser.add_argument("op", choices=ops, help="operation")
  parser.add_argument("--i_cid", dest="ifile_cid", help="input CIDs")
  parser.add_argument("--i_tid", dest="ifile_tid", help="input TIDs")
  parser.add_argument("--cids", help="CIDs (comma-separated)")
  parser.add_argument("--cid2s", help="CID #2 (comma-separated)")
  parser.add_argument("--tids", help="TIDs (e.g. gene symbol) (comma-separated)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--odir", help="output dir (for auto-named GraphML subnets)")
  parser.add_argument("--cid_skip", type=int, default=0)
  parser.add_argument("--cid_nmax", type=int, default=None)
  parser.add_argument("--tid_skip", type=int, default=0)
  parser.add_argument("--tid_nmax", type=int, default=None)
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = 'http://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  cids=[]; cid2s=[];
  if args.ifile_cid:
    fin = open(args.ifile_cid)
    while True:
      line = fin.readline()
      if not line: break
      cids.append(line.strip())
  elif args.cids:
    cids = re.split('[, ]+', args.cids.strip())
  elif args.cid2s:
    cid2s = re.split('[, ]+', args.cid2s.strip())

  tids=[];
  if args.ifile_tid:
    fin = open(args.ifile_tid)
    while True:
      line = fin.readline()
      if not line: break
      tids.append(line.strip())
  elif args.tids:
    tids = re.split('[, ]+', args.tids.strip())

  t0=time.time()

  if args.op=="D2T":
    c2b2r.slap.Utils.Drug2Targets(base_url, cids, fout)

  elif args.op=="T2D":
    c2b2r.slap.Utils.Target2Drugs(base_url, tids, fout)

  elif args.op=="DTP":
    c2b2r.slap.Utils.DrugTargetPaths(base_url, cids, tids, args.cid_skip, args.cid_nmax, args.tid_skip, args.tid_nmax, args.odir, fout)

  elif args.op=="DSS":
    c2b2r.slap.Utils.Drug2BioSimilarDrugs(base_url, cids, fout)

  elif args.op=="DDS":
    c2b2r.slap.Utils.DDSimilarity(base_url, cid, cid2, fout)

  else:
    parser.error('Unknown operation: %s'%args.op)

  logging.info('Elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))))



#!/usr/bin/env python3
"""
Utility functions for the NLM ChemIDplus REST API.
https://chem.nlm.nih.gov/chemidsearch/api
https://chem.nlm.nih.gov/api/swagger-ui.html
https://chem.nlm.nih.gov/api/v2/api-docs
"""
###
import sys,os,io,re,csv,json,time,logging,tempfile,tqdm
import requests
import urllib.request,urllib.parse
import pandas as pd
#
#
API_HOST='chem.nlm.nih.gov'
API_BASE_PATH='/api'
BASE_URL="https://"+API_HOST+API_BASE_PATH
#
# 
#############################################################################
def ListSources(base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  response = requests.get(base_url+f"/data/meta/sources")
  sources = response.json()
  for source in sources:
    if not tags: tags = list(source.keys())
    df_this = pd.DataFrame({tag:[source[tag] if tag in source else ""] for tag in tags})
    if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=1
  logging.info(f"Output records: {n_out}")
  return df

#############################################################################
def ListTypes(base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  response = requests.get(base_url+f"/data/meta/types")
  types = response.json()
  for type_this in types:
    if not tags: tags = list(type_this.keys())
    df_this = pd.DataFrame({tag:[type_this[tag] if tag in type_this else ""] for tag in tags})
    if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=1
  logging.info(f"Output records: {n_out}")
  return df

#############################################################################
def GetId2Data(ids, id_type, datatype, base_url=BASE_URL, fout=None):
  n_out=0; n_notfound=0; n_err=0; df=None;
  for i in tqdm.tqdm(range(len(ids))):
    id_this = ids[i]
    url = (base_url+f"/data/{id_type}/equals/{id_this}?data={datatype}&format=tsv")
    response = requests.get(url)
    if response.status_code!=requests.codes.ok:
      logging.info(f"HTTP status_code: {response.status_code}; ID: \"{id_this}\"")
    if response.status_code==requests.codes.not_found:
      n_notfound+=1
      continue
    df_this = pd.read_csv(io.StringIO(response.text), sep='\t')
    if datatype != "summary":
      df_this = df_this.drop("Last Modified", 1).drop(0, 0) #Drop Last Modified col, row
    if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=df_this.shape[0]
  logging.info(f"Input IDs: {len(ids)}; not_found: {n_notfound}; output records: {n_out}")
  return df

#############################################################################
def GetId2Summary(ids, id_type, base_url=BASE_URL, fout=None):
  return GetId2Data(ids, id_type, "summary", base_url, fout)

#############################################################################
def GetId2Names(ids, id_type, base_url=BASE_URL, fout=None):
  return GetId2Data(ids, id_type, "names", base_url, fout)

#############################################################################
def GetId2Numbers(ids, id_type, base_url=BASE_URL, fout=None):
  return GetId2Data(ids, id_type, "numbers", base_url, fout)

#############################################################################
def GetId2ToxicityList(ids, id_type, base_url=BASE_URL, fout=None):
  return GetId2Data(ids, id_type, "toxicityList", base_url, fout)



#!/usr/bin/env python3
'''
Commonly used functions for database client applications.
'''
import sys,os,logging,urllib.parse
import sqlalchemy

##############################################################################
def PostgreSqlConnect(dbhost, dbport, dbname, dbusr, dbpw):
  import psycopg2
  engine = sqlalchemy.create_engine(f"postgresql+psycopg2://{dbusr}:{urllib.parse.quote_plus(dbpw)}@{dbhost}:{dbport}/{dbname}")
  return engine.connect()

##############################################################################
def MySqlConnect(dbhost, dbport, dbname, dbusr, dbpw):
  try:
    # pip install MySQL-python
    # import mysql-python #default
    engine = sqlalchemy.create_engine(f"mysql://{dbusr}:{urllib.parse.quote_plus(dbpw)}@{dbhost}:{dbport}/{dbname}")
  except:
    try:
      # pip install mysql-connector-python
      # import mysql-connector-python
      engine = sqlalchemy.create_engine(f"mysql+mysqlconnector://{dbusr}:{urllib.parse.quote_plus(dbpw)}@{dbhost}:{dbport}/{dbname}")
    except:
      try:
        # pip install PyMySQL
        # import pymysql
        engine = sqlalchemy.create_engine(f"mysql+pymysql://{dbusr}:{urllib.parse.quote_plus(dbpw)}@{dbhost}:{dbport}/{dbname}")
      except:
        try:
          # pip install MySQLdb
          # import MySQLdb
          engine = sqlalchemy.create_engine(f"mysql+mysqldb://{dbusr}:{urllib.parse.quote_plus(dbpw)}@{dbhost}:{dbport}/{dbname}")
        except:
          logging.error("Failed to connect.")
          return None

  return engine.connect()



#!/usr/bin/env python3
"""
https://mor.nlm.nih.gov/download/rxnav/RxNormAPIs.html
https://rxnav.nlm.nih.gov/RxNormAPIs.html
https://rxnav.nlm.nih.gov/RxNormAPIREST.html
https://www.nlm.nih.gov/research/umls/rxnorm/docs/

 TERM TYPES
 TTY	Name
 IN	Ingredient
 PIN	Precise Ingredient
 MIN	Multiple Ingredients
 SCDC	Semantic Clinical Drug Component
 SCDF	Semantic Clinical Drug Form
 SCDG	Semantic Clinical Dose Form Group
 SCD	Semantic Clinical Drug
 GPCK	Generic Pack
 BN	Brand Name
 SBDC	Semantic Branded Drug Component
 SBDF	Semantic Branded Drug Form
 SBDG	Semantic Branded Dose Form Group
 SBD	Semantic Branded Drug
 BPCK	Brand Name Pack
 PSN	Prescribable Name
 SY	Synonym
 TMSY	Tall Man Lettering Synonym
 DF	Dose Form
 ET	Dose Form Entry Term
 DFG	Dose Form Group
"""
###
import sys,os,re,argparse,time,logging

from .. import rxnorm
from ..util import rest
#
##############################################################################
if __name__=='__main__':
  epilog='''\
get_id2rxcui requires --idtype.
get_spellingsuggestions requires --name.
get_class_members requires --class_id and --rel_src.
Example names: "prozac", "tamiflu", "metformin".
Example RXCUIs: 131725, 213269.
Example IDs: C2709711 (UMLSCUI).
Note that RxCUIs generally refer to drug products. Term Types
"Ingredient" (IN) and "Precise Ingredient" (PIN) generally refer
to chemical compounds.
'''
  parser = argparse.ArgumentParser(description='RxNorm API client utility', epilog=epilog)
  ops = [
	"version", "rawquery",
	"list_sourcetypes", "list_relationtypes", "list_termtypes",
	"list_propnames", "list_propcategories", "list_idtypes",
	"list_class_types", "list_classes",
	"get_name", "get_name2rxcui",
	"get_id2rxcui",
	"get_rxcui_status", "get_rxcui_properties", "get_rxcui_allproperties",
	"get_rxcui_ndcs", "get_rxcui_allrelated",
	"get_rxcui_classes",
	"get_class_members", "get_spellingsuggestions"
	]
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="external drug IDs or RXCUIs")
  parser.add_argument("--ids", help="drug names, IDs, or RXCUIs (comma-separated)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--idtype", help="drug ID type")
  parser.add_argument("--rel_src", help="relationship source")
  parser.add_argument("--class_id", help="drug class ID")
  parser.add_argument("--class_types", help="drug class (e.g. 'MESHPA', 'MESHPA,ATC1-4')")
  parser.add_argument("--atc", help="ATC drug classes")
  parser.add_argument("--meshpa", help="MeSH pharmacologic actions (classtype 'MESHPA'")
  parser.add_argument("--ndfrt_type", choices=rxnorm.NDFRT_TYPES, help="NDFRT drug class types")
  parser.add_argument("--tty", help="term type")
  parser.add_argument("--api_host", default=rxnorm.API_HOST)
  parser.add_argument("--api_base_path", default=rxnorm.API_BASE_PATH)
  parser.add_argument("--api_key")
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL='https://'+args.api_host+args.api_base_path

  t0=time.time()

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout
  
  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    logging.info(f'input IDs: {len(ids)}')
    fin.close()
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids.strip())
  else:
    ids=[]

  if re.match(r'get_', args.op) and not ids:
    parser.error(f'{args.op} requires --i or --ids')

  if args.op == 'version':
    rval = rest.GetURL(BASE_URL+'/version.json', parse_json=True)
    print(rval['version'])

  elif args.op == 'list_idtypes':
    rxnorm.List_IDTypes(BASE_URL, fout)

  elif args.op == 'list_sourcetypes':
    rxnorm.List_SourceTypes(BASE_URL, fout)

  elif args.op == 'list_relationtypes':
    rxnorm.List_RelationTypes(BASE_URL, fout)

  elif args.op == 'list_termtypes':
    rxnorm.List_TermTypes(BASE_URL, fout)

  elif args.op == 'list_propnames':
    rxnorm.List_PropNames(BASE_URL, fout)

  elif args.op == 'list_propcategories':
    rxnorm.List_PropCategories(BASE_URL, fout)

  elif args.op == 'list_class_types':
    rxnorm.List_ClassTypes(BASE_URL, fout)

  elif args.op == 'list_classes':
    class_types = re.split(r'[,\s]+', args.class_types.strip()) if args.class_types else None
    rxnorm.List_Classes(class_types, BASE_URL, fout)

  elif args.op == 'get_id2rxcui':
    if not args.idtype: parser.error(f'{args.op} requires --idtype')
    rxnorm.Get_ID2RxCUI(ids, args.idtype, BASE_URL, fout)

  elif args.op == 'get_name':
    rxnorm.Get_Name(ids, BASE_URL, fout)

  elif args.op == 'get_name2rxcui':
    rxnorm.Get_Name2RxCUI(ids, BASE_URL, fout)

  elif args.op == 'get_rxcui_status':
    rxnorm.Get_RxCUI_Status(ids, BASE_URL, fout)

  elif args.op == 'get_rxcui_properties':
    rxnorm.Get_RxCUI_Properties(ids, BASE_URL, fout)

  elif args.op == 'get_rxcui_allproperties':
    rxnorm.Get_RxCUI_AllProperties(ids, BASE_URL, fout)

  elif args.op == 'get_rxcui_ndcs':
    rxnorm.Get_RxCUI_NDCs(ids, BASE_URL, fout)

  elif args.op == 'get_rxcui_allrelated':
    rxnorm.Get_RxCUI_AllRelated(ids, BASE_URL, fout)

  elif args.op == "get_rxcui_classes":
    rxnorm.Get_RxCUI_Classes(ids, BASE_URL, fout)

  elif args.op == 'get_class_members':
    rxnorm.Get_Class_Members(ids, BASE_URL, fout)

  elif args.op == 'get_spellingsuggestions':
    rxnorm.Get_SpellingSuggestions(ids, BASE_URL, fout)

  elif args.op == 'rawquery':
    rxnorm.RawQuery(args.rawquery, BASE_URL, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

  logging.info(f"Elapsed time: {time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))}")



#!/usr/bin/env python3
"""
Monarch REST utilities.

https://monarchinitiative.org/page/services
https://monarchinitiative.org/docs/files/api-js.html
https://github.com/monarch-initiative/owlsim-v3
https://github.com/monarch-initiative/monarch-analysis

/search	searches over ontology terms via OntoQuest
/autocomplete	proxy for vocbaulary services autocomplete
/disease	disease info or page
/phenotype	phenotype info or page
/simsearch	OwlSim search using search profile of ontology classes

https://monarchinitiative.org/disease/OMIM_127750.json
https://monarchinitiative.org/phenotype/HP_0000003.json
https://monarchinitiative.org/compare/OMIM:270400/NCBIGene:5156,OMIM:249000,OMIM:194050.json

We use OwlSim for semantic matching when comparing two entities (such as genes,
diseases, or genotypes) with sets of attributes (phenotypes). URLs are of the form
/compare/:x/:y.json
where x can be either an entity or a list of phenotypes and y can be a list of
entities and/or sets of phenotypes.
This wraps a call to getGroupwiseSimilarity(x,y) in OwlSim. This works such that
given a query id (such as a gene, genotype, disease), and one or more target
identifiers, it will map each to it's respective phenotypes, and perform an OwlSim
comparison of the query to each target. You are permitted to mix query and target
types. For example, your query can be a disease, and the target(s) be a list of
gene(s), disease(s), phenotype(s), and/or genotype(s). You can indicate to union the
phenotypes of either the query or the target with a plus "+". Only one entity may be
supplied for the query, whereas multiple target entities are allowed (delimited by a
comma).  For details on owlsim, see http://owlsim.org

Paths:
/compare/:id1/:id2
/compare/:id1/:id2,id3,...idN
/compare/:id1+:id2/:id3,:id4,...idN
/compare/:id1/:id2+:id3,:id4,:id5+:id6,...,:idN

IC = Information Content
LCS = Least Common Subsumers
See OwlSim docs and pubs.
https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3653101/
"""
###
import sys,os,re,json,logging
import pandas as pd
import urllib,urllib.parse
#
from ..util import rest
#
API_HOST="monarchinitiative.org"
API_BASE_PATH=""
BASE_URL = "https://"+API_HOST+API_BASE_PATH
#
##############################################################################
def GetDisease(ids, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  for id_this in ids:
    disease = rest.GetURL(f"{base_url}/disease/{id_this}.json", parse_json=True)
    logging.debug((json.dumps(disease, indent=2, sort_keys=False)))
    if not tags:
      tags = list(disease.keys())
      for tag in ("relationships", "equivalentNodes", "equivalentClasses", "bundleJS", "bundleCSS"):
        tags.remove(tag)
    df = pd.concat([df, pd.DataFrame({tags[j]:[disease[tags[j]]] for j in range(len(tags))})])
  logging.info(f"IDs in: {len(ids)}; n_out: {df.shape[0]}")
  if fout is not None: df.to_csv(fout, "\t", index=False)
  else: return df

##############################################################################
def GetDiseaseRelationships(ids, base_url=BASE_URL, fout=None):
  tags_dis=[]; tags_rel=[]; df=pd.DataFrame();
  for id_this in ids:
    disease = rest.GetURL(f"{base_url}/disease/{id_this}.json", parse_json=True)
    logging.debug((json.dumps(disease, indent=2, sort_keys=False)))
    if not tags_dis:
      for tag in disease.keys():
        if type(disease[tag]) not in (list, dict): tags_dis.append(tag)
    rels = disease["relationships"] if "relationships" in disease else []
    for rel in rels:
      if not tags_rel:
        tags_rel = list(rel.keys())
      df_this = pd.concat([
        pd.DataFrame({tags_dis[j]:[disease[tags_dis[j]]] for j in range(len(tags_dis))}),
        pd.DataFrame({tags_rel[j]:[rel[tags_rel[j]]] for j in range(len(tags_rel))}) ], axis=1)
      df = pd.concat([df, df_this], axis=0)
  logging.info(f"IDs in: {len(ids)}; n_out: {df.shape[0]}")
  if fout is not None: df.to_csv(fout, "\t", index=False)
  else: return df

##############################################################################
def GetPhenotype(ids, base_url=BASE_URL, fout=None):
  return

##############################################################################
def GetGene(ids, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  for gid in ids:
    gene = rest.GetURL(f"{base_url}/gene/{gid}.json", parse_json=True)
    logging.debug((json.dumps(gene, indent=2, sort_keys=False)))
    if not tags: tags = list(gene.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[gene[tags[j]]] for j in range(len(tags))})])
  logging.info(f"IDs in: {len(ids)}; n_out: {df.shape[0]}")
  if fout is not None: df.to_csv(fout, "\t", index=False)
  else: return df

##############################################################################
### Guess: The matches each consist of a 3-tuple,
###	A = phenotype in profileA,
###	B = phenotype in profileB, and
###	LCS = Least Common Subsumers phenotype in...
### EPO?  (But we see MP, ZP, HP...)
##############################################################################
def ComparePhenotypes(idAs, idBs, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  #fout.write("idA,typeA,labelA,taxonA,idB,typeB,labelB,taxonB,url,matchidA,matchlabelA,matchidB,matchlabelB,matchidLCS,matchlabelLCS,matchicA,matchicB,matchicLCS\n")
  for idA in idAs:
    url_this = f"{base_url}/compare/{idA}/{(','.join(idBs))}.json"
    rval = rest.GetURL(url_this, parse_json=True)
    logging.debug(json.dumps(rval, indent=2, sort_keys=False))
    cmpr = rval
    if not tags: tags = list(cmpr.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[cmpr[tags[j]]] for j in range(len(tags))})])

#    A = cmpr["a"] if "a" in cmpr else {}
#    typeA = A["type"] if "type" in A else ""
#    idAx = A["id"] if "id" in A else ""
#    labelA = A["label"] if "label" in A else ""
#    taxonA = A["taxon"]["id"] if ("taxon" in A and "id" in A["taxon"]) else {}
#    logging.info("A[type]: "%s" ; A[id]: "%s" ; A[label]: "%s""%(typeA, idA, labelA))
#    resource = cmpr["resource"] if "resource" in cmpr else {}
#    for key in resource.keys():
#      logging.info("resource[%s]: %s"%(key,resource[key]))
#    metadata = cmpr["metadata"] if "metadata" in cmpr else {}
#    for key in metadata.keys():
#      logging.info("metadata[%s]: %s"%(key,str(metadata[key])))
#    Bs = cmpr["b"] if "b" in cmpr else []
#    for B in Bs:
#      labelB = B["label"] if "label" in B else ""
#      typeB = B["type"] if "type" in B else ""
#      idB = B["id"] if "id" in B else ""
#      logging.info('B[type]: "%s" ; B[id]: "%s" ; B[label]: "%s"'%(typeB, idB, labelB))
#      bscore = B["score"] if "score" in B else {}
#      taxonB = B["taxon"]["id"] if ("taxon" in B and "id" in B["taxon"]) else {}
#      matches = B["matches"] if "matches" in B else []
#      vals_ab=[idA,typeA,labelA,taxonA,idB,typeB,labelB,taxonB,url_this]
#      for m in matches:
#        n_match+=1
#        mA = m["a"] if "a" in m else {}
#        matchidA = mA["id"] if "id" in mA else ""
#        matchlabelA = mA["label"] if "label" in mA else ""
#        mB = m["b"] if "b" in m else {}
#        matchidB = mB["id"] if "id" in mB else ""
#        matchlabelB = mB["label"] if "label" in mB else ""
#        mLCS = m["lcs"] if "lcs" in m else {}
#        matchidLCS = mLCS["id"] if "id" in mLCS else ""
#        matchlabelLCS = mLCS["label"] if "label" in mLCS else ""
#        matchicA = mA["IC"] if "IC" in mA else "" #same as LCS?
#        matchicB = mB["IC"] if "IC" in mB else "" #same as LCS?
#        matchicLCS = mLCS["IC"] if "IC" in mLCS else ""
#        vals_this=vals_ab+[matchidA,matchlabelA,matchidB,matchlabelB,matchidLCS,matchlabelLCS,matchicA,matchicB,matchicLCS]
#        fout.write("\t".join([str(val) for val in vals_this])+"\n")
#        n_out+=1

  logging.info(f"n_comparison = {len(idAs)*len(idBs)} ({len(idAs)}x{len(idBs)})")
  #logging.info("n_match = {n_match}; n_out = {n_out}")
  logging.info(f"n_out: {df.shape[0]}")
  if fout is not None: df.to_csv(fout, "\t", index=False)
  else: return df

##############################################################################



#!/usr/bin/env python3
###

import sys,os,time,argparse,logging
import numpy as np
import pandas as pd
import h5py

###
def h5_type(ob):
  return "Group" if type(ob) is h5py.Group else "Dataset" if type(ob) is h5py.Dataset else type(ob)

###
def list_all(name, ob):
  logging.info(f"{ob.name} ({h5_type(ob)})")
  if type(ob) is h5py.Dataset:
    logging.info(f"Dataset dtype: {ob.dtype}; shape: {ob.shape}; size: {ob.size}; ndim: {ob.ndim}; nbytes: {ob.nbytes}")
  return None

#############################################################################
def Summary(f):
  logging.debug(list(f.keys()))
  for k in list(f.keys()):
    if type(f[k]) is h5py.Group:
      logging.debug(f"{f.name}:{f[k].name}")
      f[k].visititems(list_all)
    else:
      logging.info(f"{f[k].name} ({h5_type(f[k])})")

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='H5 file operations', epilog="")
  OPS = ['summary']
  parser.add_argument("op", choices=OPS, help='OPERATION')
  parser.add_argument("--i", dest="ifile", required=True, help="input file")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  t0 = time.time();

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  finh5 = h5py.File(args.ifile, 'r')

  if args.op == "summary":
    Summary(finh5)

  else:
    parser.error(f"Unsupported operation: {args.op}")

  logging.info(f"Elapsed: {time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))}")




#!/usr/bin/env python3
"""
Access to Uniprot REST API.
https://www.uniprot.org/help/api
UniprotKB = Uniprot Knowledge Base

python3 -m BioClients.uniprot.Client --uids Q14790 getData
"""
import sys,os,re,argparse,time,logging
#
from .. import uniprot
#
API_HOST='www.uniprot.org'
API_BASE_PATH='/uniprot'
#
##############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='Uniprot query client; get data for specified IDs')
  ops = ['getData', 'listData']
  ofmts = ['txt', 'tab', 'xml', 'rdf', 'fasta', 'gff']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--uids", dest="uids", help="UniProt IDs, comma-separated (ex: Q14790)")
  parser.add_argument("--i", dest="ifile", help="input file, UniProt IDs")
  parser.add_argument("--o", dest="ofile", help="output (CSV)")
  parser.add_argument("--ofmt", default='txt')
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URI='https://'+args.api_host+args.api_base_path

  if args.ofile:
    fout = open(args.ofile, 'w')
  else:
    fout = sys.stdout

  t0=time.time()

  uids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      uids.append(line.strip())
  elif args.uids:
    uids = re.split(r'[\s,]+', args.uids.strip())
  else:
    parser.error('--i or --uids required.')

  if args.op == 'getData':
    uniprot.GetData(BASE_URI, uids, args.ofmt, fout)

  else:
    parser.error('Unknown operation: %s'%args.op)

  logging.info('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))




#!/usr/bin/env python3
"""
	XML utility functions.
"""
import sys,os,re,gzip,argparse,logging

from xml.etree import ElementTree
from xml.parsers import expat

#############################################################################
def GetLeafValsByTagName(root, tag):
  vals=[]
  for node in root.iter(tag):
    txt=node.text
    if txt: vals.append(txt)
  return vals

#############################################################################
def GetAttr(root, tag, attname):
  vals=[]
  for node in root.iter(tag):
    if attname in node.attrib:
      vals.append(node.attrib[attname])
  return vals

#############################################################################
def GetNodeAttr(node, attname):
  return node.attrib[attname] if attname in node.attrib else None

#############################################################################
def Describe(fin):
  n_elem=0; n_term=0; tags={};
  et_itrabl = ElementTree.iterparse(fin, events=("start", "end"))
  for event,elem in et_itrabl:
    n_elem+=1
    if event=='start':
      if elem.tag not in tags:
        tags[elem.tag]=0
      tags[elem.tag]+=1
    logging.debug('event:{}, elem.tag:{}, elem.text:{}'.format(event, elem.tag, elem.text))
    for k,v in elem.attrib.items():
      logging.debug('elem.attrib["{}"]: {}'.format(k, v))
    n_term+=1
  logging.debug('n_elem: {}; n_term: {}'.format(n_elem, n_term))
  for tag in sorted(tags.keys()):
    logging.info('{}: {}'.format(tag, tags[tag]))

#############################################################################
def XML2TSV(xp, ns, fin, fout):
  n_entity=0; tags=[];
  tree = ElementTree.parse(fin)
  logging.debug('root.tag <{}>'.format(tree.getroot().tag))
  itr = tree.iterfind(xp)
  for elem in itr:
    logging.debug('elem.tag <{}>'.format(elem.tag))
    n_entity+=1
    vals={};
    if not tags:
      tags = [re.sub(r'^{.*}', '', se.tag) for se in list(elem)]
      tags = sorted(tags)
      fout.write("\t".join(tags)+"\n")
    for subelem in list(elem):
      val = subelem.text.strip() if subelem.text else ""
      logging.debug('subelem.tag <{}>; text=\"{}\"'.format(subelem.tag, val))
      vals[re.sub(r'^{.*}', '', subelem.tag)] = val
    fout.write("\t".join([vals[tag] if tag in vals else "" for tag in tags])+"\n")
  logging.info('n_entity <{}>: {}'.format(xp, n_entity))

#############################################################################
def MatchXPath(xp, ns, fin):
  n_node=0;
  fout.write("xpath\tmatch\n")
  tree = ElementTree.parse(fin)
  itr = tree.iterfind(xp)
  for node in itr:
    n_node+=1;
    txt = node.text if node.text else ''
    fout.write("{}\t{}\n".format(xp, txt))
  logging.info("matches: {}".format(n_node))

#############################################################################
if __name__=="__main__":
  parser = argparse.ArgumentParser(description="XML utility", epilog="clean: UTF-8 encoding compliant")
  ops = ["match_xpath", "describe", "clean", "xml2tsv"]
  parser.add_argument("op", choices=ops, help="operation")
  parser.add_argument("--i", dest="ifile", help="input XML file")
  parser.add_argument("--xpath", help="xpath pattern")
  parser.add_argument("--namespace", help="XML namespace")
  parser.add_argument("--force", type=bool, help="ignore UTF-8 encoding errors")
  parser.add_argument("--o", dest="ofile", help="output XML file")
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format="%(levelname)s:%(message)s", level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  codecs_mode = "ignore" if args.force else "replace"

  if args.ifile:
    if re.match(r'^.*\.gz$', args.ifile):
      fin = gzip.open(args.ifile)
    else:
      fin = open(args.ifile, "r")
  else:
    fin = sys.stdin

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  if args.op == "describe":
    Describe(fin)

  elif args.op == "clean":
    root = ElementTree.parse(fin)
    fin.close()
    root.write(fout, encoding="UTF-8", xml_declaration=True)

  elif args.op == "match_xpath":
    if not (args.xpath and args.ifile):
      parser.error("--xpath required.")
    MatchXPath(args.xpath, args.namespace, fin) 

  elif args.op == "xml2tsv":
    if not args.xpath:
      parser.error("--xpath required.")
    XML2TSV(args.xpath, args.namespace, fin, fout)

  else:
    parser.error("Invalid operation. %s"%(args.op))



#!/usr/bin/env python3
"""
https://mygene.info/
https://mygene.info/v3/api
https://pypi.org/project/mygene/
"""
###
#
import sys,os
import pandas as pd
import mygene as mg
#
FIELDS = 'HGNC,symbol,name,taxid,entrezgene,ensemblgene'
NCHUNK=100;
#
#############################################################################
def GetGenes(ids, fields=FIELDS, fout=None):
  """Get genes by Entrez or Ensembl gene ID."""
  ichunk=0; n_out=0; df=None;
  mgi = mg.MyGeneInfo()
  while ichunk*NCHUNK<len(ids):
    df_this = mgi.getgenes(ids[ichunk*NCHUNK:((ichunk+1)*NCHUNK)], fields, as_dataframe=True)
    if fout is not None: df.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=df_this.shape[0]
    ichunk+=1
  return df

#############################################################################
def SearchGenes(queries, species, fout=None):
  """Search genes by symbol, etc. using MyGene syntax."""
  ichunk=0; n_out=0; df=None;
  mgi = mg.MyGeneInfo()
  for qry in queries:
    df_this = mgi.query(qry, species=species, as_dataframe=True)
    if fout is not None: df.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=df_this.shape[0]
    ichunk+=1
  return df

#############################################################################



#!/usr/bin/env python3
#############################################################################
### - Disease Ontology REST client 
#############################################################################
import sys,os,re,json,argparse,time,logging
#
from ..util import rest
#
API_HOST='www.disease-ontology.org'
API_BASE_PATH='/api'
#
OFMTS={'XML':'application/xml','JSON':'application/json'}
#
HEADERS={'Accept':OFMTS['JSON']}
#
##############################################################################
def GetMetadata(ids,base_url,fout):
  n_out=0;
  fout.write('doid,name,definition,child_count,parent_count,meshid\n')
  for doid in ids:
    try:
      rval=rest.GetURL(base_url+'/metadata/%s'%(doid),headers=HEADERS,parse_json=True)
    except Exception as e:
      logging.error('%s'%(e))
      continue
    if not type(rval)==types.DictType:
      logging.info('Error: %s'%(str(rval)))
      continue

    name = rval['name'] if rval.has_key('name') else ''
    definition = rval['definition'] if rval.has_key('definition') else ''
    parents = rval['parents'] if rval.has_key('parents') else []
    children = rval['children'] if rval.has_key('children') else []
    synonyms = rval['synonyms'] if rval.has_key('synonyms') else []
    subsets = rval['subsets'] if rval.has_key('subsets') else []
    xrefs = rval['xrefs'] if rval.has_key('xrefs') else []
    meshid=''
    for xref in xrefs:
      if re.match('MSH:',xref): meshid=xref[4:]
      break
    fout.write('"%s","%s","%s",%d,%d,"%s"\n'%(doid,name,definition,len(children),len(parents),meshid))
    n_out+=1

    logging.info((json.dumps(rval,indent=2,sort_keys=False)))
  logging.info('DOIDs in: %d ; records out: %d'%(len(ids),n_out))

##############################################################################
def ListTopClasses(base_url,fout):
  n_out=0;
  rval=rest.GetURL(base_url+'/metadata/DOID:4',headers=HEADERS,parse_json=True)
  children = rval['children'] if rval.has_key('children') else []
  for c in children:
    desc,doid = c
    fout.write('\t%-14s:\t"%s"\n'%(doid,desc))

##############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description="Disease Ontology REST client")
  ops = ['get_metadata', 'list_topclasses']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--ids", help="query DOIDs, comma-separated")
  parser.add_argument("--i", dest="ifile", help="query file, DOIDs")
  parser.add_argument("--o", dest="ofile", help="output (CSV|TSV)")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  base_url='https://'+args.api_host+args.api_base_path

  if len(set(fields_vals) - set(fields_allowed))>0:
    parser.error('disallowed fields: %s'%str(set(fields_vals) - set(fields_allowed)))

  if args.ofile:
    fout=open(args.ofile,"w")
    if not fout: parser.error('cannot open outfile: %s'%args.ofile)
  else:
    fout=sys.stdout

  t0=time.time()

  ids=[];
  if args.ifile:
    fin=open(args.ifile)
    if not fin: parser.error('cannot open args.ifile: %s'%args.ifile)
    while True:
      line=fin.readline()
      if not line: break
      try:
        doid=line.rstrip()
      except Exception as e:
        logging.error('invalid DOID: "%s"'%(line.rstrip()))
        continue
      if line.rstrip(): ids.append(doid)
    logging.info('input queries: %d'%(len(ids)))
    fin.close()
  elif args.ids:
    ids = re.split('[,\s]+', args.ids.strip())

  if args.op == "get_metadata":
    GetMetadata(ids, base_url,fout)

  elif args.op == "list_topclasses":
    ListTopClasses( base_url,fout)

  else:
    parser.error("No operation specified.")

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
"""
 Online Mendelian Inheritance in Man,
 "An Online Catalog of Human Genes and Genetic Disorders"

 See: https://omim.org/help/api
 The OMIM API URLs are organized in a very simple fashion:
   /api/[handler]?[parameters]
   /api/[handler]/[component]?[parameters]
   /api/[handler]/[action]?[parameters]
 The handler refers to the data object, such as an entry or a clinical synopsis.

 Handlers: entry, clinicalSynopsis, geneMap, search, html, dump

 entry?mimNumber=100100&format=json&apiKey=...

 https://omim.org/downloads/1kweo1_nTfuTCiVRnANNGQ
"""
###
import sys,os,re,json,argparse,time,logging
import urllib,urllib.parse
#
from ..util import rest
#
API_HOST='api.omim.org'	#US
#API_HOST='api.europe.omim.org'	#Europe
API_BASE_PATH='/api'
API_KEY='1kweo1_nTfuTCiVRnANNGQ' #will expire on Nov. 19th, 2019.
#
##############################################################################
if __name__=='__main__':
  fields_allowed=['text', 'existflags', 'allelicVariantList', 'clinicalSynopsis', 'seeAlso', 'referenceList', 'geneMap', 'externalLinks', 'contributors', 'creationDate', 'editHistory', 'dates', 'all' ]
  epilog='''\
operations: {
        entry : "entry resource, fetch records",
        clinical : "clinicalSynopsis resource, fetch records",
        genemap : "geneMap resource, fetch records",
        search : "search via defined syntax, some/all fields" },
example searches: {
	"muscular AND dystrophy NOT duchenne",
	"(muscular AND dystrophy) OR (duchenne AND gene)",
	"Bombay syndrome" }
'''
  parser = argparse.ArgumentParser(
        description="OMIM API client (Online Mendelian Inheritance in Man)", epilog=epilog)
  ops = ['entry', 'clinical', 'genemap', 'search']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--o", dest="ofile", help="output (CSV|TSV)")
  parser.add_argument("--mims", help="MIM ID[s], comma-separated (e.g. 100100,100200)")
  parser.add_argument("--query", help="search query")
  parser.add_argument("--rawquery", help="search rawquery")
  parser.add_argument("--inc_fields", default="all",  help="specify fields, comma-separated, or 'all'")
  parser.add_argument("--fmt", default="json", choices=('json', 'xml', 'python'), help="output format")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("--api_key", default=API_KEY)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  if args.mims:
    mim_vals = map(lambda n:int(n), re.split(',', args.mims))
  else:
    parser.error("--mims required.")

  fields_vals = re.split(',', args.inc_fields)

  if len(set(fields_vals) - set(fields_allowed))>0:
    parser.error('disallowed fields: %s'%str(set(fields_vals) - set(fields_allowed)))

  if args.ofile:
    fout=open(args.ofile,"w")
    if not fout: parser.error('cannot open outfile: %s'%args.ofile)
  else:
    fout=sys.stdout

  t0=time.time()

  API_BASE_URL='https://'+args.api_host+args.api_base_path
  url=API_BASE_URL

  url_params=('&include=%s'%args.inc_fields if args.inc_fields else '')
  url_params+=('&format=%s'%args.fmt)
  url_params+=('&apiKey=%s'%args.api_key)

  if args.op == 'entry':
    url=API_BASE_URL+'/entry?mimNumber=%s'%(','.join(map(lambda n:str(n), mim_vals)))+url_params
    rval=rest.GetURL(url, parse_json=True, verbose=args.verbose)
    fout.write(json.dumps(rval, sort_keys=True, indent=2))

  elif args.op == 'genemap':
    url=API_BASE_URL+'/geneMap?mimNumber=%s'%(','.join(map(lambda n:str(n), mim_vals)))+url_params
    rval=rest.GetURL(url, parse_json=True, verbose=args.verbose)
    fout.write(json.dumps(rval, sort_keys=True, indent=2))

  elif args.op == 'clinical':
    url=API_BASE_URL+'/clinicalSynopsis?mimNumber=%s'%(','.join(map(lambda n:str(n), mim_vals)))+url_params
    rval=rest.GetURL(url, parse_json=True, verbose=args.verbose)
    fout.write(json.dumps(rval, sort_keys=True, indent=2))

  elif args.op == 'search':
    url=API_BASE_URL+'/entry/search?search=%s'%(urllib.parse.quote(query))+url_params
    rval=rest.GetURL(url, parse_json=True)
    fout.write(json.dumps(rval, sort_keys=True, indent=2))

  elif args.op == 'rawquery':
    url=API_BASE_URL+rawquery
    rval=rest.GetURL(url, parse_json=True)
    fout.write(json.dumps(rval, sort_keys=True, indent=2))

  else:
    parser.error('Unknown operation: %s'%args.op)

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
"""
Pharos  REST API client
https://pharos.nih.gov/idg/api/v1/targets(589)
"""
###
import sys,os,argparse,json,re,time,logging
#
from .. import idg
#
#############################################################################
if __name__=='__main__':
  API_HOST="pharos.nih.gov"
  API_BASE_PATH="/idg/api/v1"
  IDTYPES = ['IDG_TARGET_ID', 'UNIPROT', 'ENSP', 'GSYMB']
  parser = argparse.ArgumentParser(description='Pharos REST API client')
  ops = [ 'list_targets', 'list_ligands', 'list_diseases',
	'get_targets', 'get_targetProperties', 'search_targets' ]
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input file, target IDs")
  parser.add_argument("--ids", dest="ids", help="IDs, target, comma-separated")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--idtype", choices=IDTYPES, default='IDG_TARGET_ID', help="target ID type")
  parser.add_argument("--nmax", type=int, help="max to return")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    logging.info('input IDs: %d'%(len(ids)))
    fin.close()
  elif args.ids:
    ids = re.split(r'\s*,\s*',args.ids.strip())

  t0 = time.time()

  if re.match(r'^get_', args.op) and not ids:
    parser.error('{0} requires IDs.'.format(args.op))

  if args.op=='get_targets':
    idg.GetTargets(BASE_URL, ids, args.idtype, fout)

  elif args.op=='get_targetProperties':
    idg.GetTargetProperties(BASE_URL, ids, args.idtype, fout)

  elif args.op=='list_targets':
    idg.ListItems('targets', BASE_URL, fout)

  elif args.op=='list_diseases':
    idg.ListItems('diseases', BASE_URL, fout)

  elif args.op=='list_ligands':
    idg.ListItems('ligands', BASE_URL, fout)

  elif args.op=='search_targets':
    logging.error('Not implemented yet.')

  else:
    logging.error('Invalid operation: {0}'.format(args.op))

  logging.info(('Elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
"""
 https://www.disgenet.org/api/
 https://www.disgenet.org/dbinfo
 DisGeNET Disease Types : disease, phenotype, group
 DisGeNET Metrics:
   GDA Score
   VDA Score
   Disease Specificity Index (DSI)
   Disease Pleiotropy Index (DPI)
   Evidence Level (EL)
   Evidence Index (EI)
 GNOMAD pLI (Loss-of-function Intolerant)

 DisGeNET Association Types:
   Therapeutic
   Biomarker
   Genomic Alterations
   GeneticVariation
   Causal Mutation
   Germline Causal Mutation
   Somatic Causal Mutation
   Chromosomal Rearrangement
   Fusion Gene
   Susceptibility Mutation
   Modifying Mutation
   Germline Modifying Mutation
   Somatic Modifying Mutation
   AlteredExpression
   Post-translational Modification
"""
###
import sys,os,io,re,time,requests,json,logging
import pandas as pd
#
API_HOST='www.disgenet.org'
API_BASE_PATH='/api'
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
SOURCES = [
	"CURATED",
	"INFERRED",
	"ANIMAL_MODELS",
	"ALL",
	"BEFREE",
	"CGI",
	"CLINGEN",
	"CLINVAR",
	"CTD_human",
	"CTD_mouse",
	"CTD_rat",
	"GENOMICS_ENGLAND",
	"GWASCAT",
	"GWASDB",
	"HPO",
	"LHGDN",
	"MGD",
	"ORPHANET",
	"PSYGENET",
	"RGD",
	"UNIPROT" ]
DISEASE_TYPES = ["disease", "phenotype", "group"]
DISEASE_CLASSES = [ # MeSH Disease classes:
 "C01", # Bacterial Infections and Mycoses
 "C02", # Virus Diseases
 "C03", # Parasitic Diseases
 "C04", # Neoplasms
 "C05", # Musculoskeletal Diseases
 "C06", # Digestive System Diseases
 "C07", # Stomatognathic Diseases
 "C08", # Respiratory Tract Diseases
 "C09", # Otorhinolaryngologic Diseases
 "C10", # Nervous System Diseases
 "C11", # Eye Diseases
 "C12", # Male Urogenital Diseases
 "C13", # Female Urogenital Diseases and Pregnancy Complications
 "C14", # Cardiovascular Diseases
 "C15", # Hemic and Lymphatic Diseases
 "C16", # Congenital, Hereditary, and Neonatal Diseases and Abnormalities
 "C17", # Skin and Connective Tissue Diseases
 "C18", # Nutritional and Metabolic Diseases
 "C19", # Endocrine System Diseases
 "C20", # Immune System Diseases
 "C21", # Disorders of Environmental Origin
 "C22", # Animal Diseases
 "C23", # Pathological Conditions, Signs and Symptoms
 "C24", # Occupational Diseases
 "C25", # Substance-Related Disorders
 "C26", # Wounds and Injuries
 "F01", # Behavior and Behavior Mechanisms
 "F02", # Psychological Phenomena
 "F03", # Mental Disorders
	]
#
##############################################################################
def GetApiKey(session, user_email, user_password, base_url=BASE_URL):
  api_key=None;
  auth_params = {"email":user_email, "password":user_password}
  url = f"{base_url}/auth/"
  logging.debug(f"URL: {url}")
  try:
    response = session.post(url, data=auth_params)
    if (response.status_code==200):
      json_response = response.json()
      api_key = json_response.get("token")
      session.headers.update({"Authorization": f"Bearer {api_key}"})
    else:
      logging.error(f"GetApiKey failed (status_code={response.status_code}): {response.text}")
  except requests.exceptions.RequestException as e:
    logging.error(f"GetApiKey failed: {e}")
  logging.debug(f"api_key: {api_key}")
  return api_key

##############################################################################
def GetVersion(user_email, user_password, base_url=BASE_URL, fout=None):
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  url = f"{base_url}/version/"
  response = session.get(url, params={'format':'tsv'})
  df = pd.read_csv(io.StringIO(response.text), sep='\t')
  if fout is not None: df.to_csv(fout, sep='\t', index=False)
  return df

##############################################################################
def GetGeneGDAs(ids, source, user_email, user_password, base_url=BASE_URL, fout=None):
  '''NCBI Entrez Identifier or HGNC Symbol'''
  tags=[]; n_out=0; df=None;
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  params = {'source': source}
  for id_this in ids:
    url = f"{base_url}/gda/gene/{id_this}"
    response = session.get(url, params=params)
    gdas = response.json()
    for gda in gdas:
      if not tags:
        tags = list(gda.keys())
        for tag in tags[:]:
          if type(gda[tag]) in (list,dict):
            tags.remove(tag)
            logging.info(f"Ignoring field: {tag}")
      df_this = pd.DataFrame({tag:[gda[tag] if tag in gda else ""] for tag in tags})
      if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
      else: df = pd.concat([df, df_this])
      n_out+=1
  session.close()
  logging.info(f"IDs: {len(ids)}; GDAs: {n_out}")
  return df

##############################################################################
def GetDiseaseGDAs(ids, source, user_email, user_password, base_url=BASE_URL, fout=None):
  tags=[]; n_out=0; df=None;
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  logging.debug(f"HEADERS: {session.headers}")
  params = {'source': source}
  for id_this in ids:
    url = f"{base_url}/gda/disease/{id_this}"
    response = session.get(url, params=params)
    gdas = response.json()
    for gda in gdas:
      if not tags:
        tags = [tag for tag in gda.keys() if type(gda[tag]) not in (list,dict)]
      df_this = pd.DataFrame({tag:[gda[tag] if tag in gda else ""] for tag in tags})
      if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
      else: df = pd.concat([df, df_this])
      n_out+=1
  logging.info(f"IDs: {len(ids)}; GDAs: {n_out}")
  return df

##############################################################################
def GetProteinGDAs(ids, source, user_email, user_password, base_url=BASE_URL, fout=None):
  """Uniprot protein accession."""
  tags=[]; n_out=0; df=None;
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  for id_this in ids:
    url = f"{base_url}/gda/uniprot/{id_this}?source={source}"
    response = requests.get(url, headers=headers)
    gdas = response.json()
    for gda in gdas:
      if not tags:
        tags = [tag for tag in gda.keys() if type(gda[tag]) not in (list,dict)]
      df_this = pd.DataFrame({tag:[gda[tag] if tag in gda else ""] for tag in tags})
      if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
      else: df = pd.concat([df, df_this])
      n_out+=1
  logging.info(f"IDs: {len(ids)}; GDAs: {n_out}")
  return df

##############################################################################
def ListDiseases(user_email, user_password, base_url=BASE_URL, fout=None):
  tags=[]; n_out=0; df=None;
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  for source in SOURCES:
    url = f"{base_url}/disease/source/{source}"
    params = {'format':'tsv'}
    response = session.get(url, params=params)
    if (response.status_code!=200):
      logging.error(f"(status_code={response.status_code}): source: {source}")
      continue
    df_this = pd.read_csv(io.StringIO(response.text), sep='\t')
    if fout is not None: df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=df_this.shape[0]
  logging.info(f"Diseases: {n_out}")

##############################################################################
def GetDiseases(source, dtype, dclass, nmax, user_email, user_password, base_url=BASE_URL, fout=None):
  tags=[]; n_out=0; df=None;
  session = requests.Session()
  api_key = GetApiKey(session, user_email, user_password, base_url)
  if api_key is None: return
  url = f"{base_url}/disease/source/{source}"
  params = {'format':'tsv'}
  if dtype is not None: params['type'] = dtype
  if dclass is not None: params['disease_class'] = dclass
  if nmax is not None: params['limit'] = nmax
  response = session.get(url, params=params)
  #logging.debug(response.text)
  df = pd.read_csv(io.StringIO(response.text), sep='\t')
  if fout is not None: df.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
  n_out+=df.shape[0]
  logging.info(f"Diseases: {n_out}")

##############################################################################



#! /usr/bin/env python3
"""
Developed and tested with doid.obo (Disease Ontology).
"""
import sys,os,argparse,re,logging

from .. import obo as util_obo

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='OBO to TSV converter')
  parser.add_argument("--i", dest="ifile", required=True, help="input OBO file")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  fin = open(args.ifile)

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  util_obo.OBO2CSV(fin, fout)



#!/usr/bin/env python3
##############################################################################
### Medline Plus Genetics
### https://medlineplus.gov/about/developers/geneticsdatafilesapi/
### https://medlineplus.gov/download/ghr-summaries.xml
##############################################################################
### https://wsearch.nlm.nih.gov/ws/query?db=ghr&term=alzheimer
### https://medlineplus.gov/download/genetics/condition/alzheimer-disease.json
##############################################################################
import sys,os,re,argparse,time,logging
#
from ... import medline
#
##############################################################################
if __name__=="__main__":
  epilog = """
Example conditions:
allergic-asthma,
alzheimer-disease,
parkinson-disease,
rapid-onset-dystonia-parkinsonism,
type-1-diabetes,
type-2-diabetes,

"""
  ops = ["search", "list_conditions", "list_genes", "get_condition_genes"]
  parser = argparse.ArgumentParser(description="MedlinePlus Genetics REST API client", epilog=epilog)
  parser.add_argument("op", choices=ops, help="OPERATION (select one)")
  parser.add_argument("--i", dest="ifile", help="input term file (one per line)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--ids", help="term list (comma-separated)")
  parser.add_argument("--api_host", default=medline.genetics.API_HOST)
  parser.add_argument("--api_base_path", default=medline.genetics.API_BASE_PATH)
  parser.add_argument("--download_host", default=medline.genetics.DOWNLOAD_HOST)
  parser.add_argument("--download_base_path", default=medline.genetics.DOWNLOAD_BASE_PATH)
  parser.add_argument("--summary_url", default=medline.genetics.SUMMARY_URL)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format="%(levelname)s:%(message)s", level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  api_base_url="https://"+args.api_host+args.api_base_path
  download_base_url="https://"+args.download_host+args.download_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  t0=time.time()

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())
  if len(ids)>0: logging.info(f"Input IDs: {len(ids)}")

  if args.op=="search":
    medline.genetics.Utils.Search(ids, api_base_url, fout)

  elif args.op=="list_conditions":
    medline.genetics.Utils.ListConditions(args.summary_url, fout)

  elif args.op=="list_genes":
    medline.genetics.Utils.ListGenes(args.summary_url, fout)

  elif args.op=="get_condition_genes":
    medline.genetics.Utils.GetConditionGenes(ids, download_base_url, fout)

  else:
    parser.error(f"Operation invalid: {args.op}")

  logging.info(f"Elapsed time: {time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))}")




#!/usr/bin/env python3
"""
TCRD MySql db client utilities.
"""
import os,sys,re,time,json,logging,yaml,tqdm
import pandas as pd

TDLS = ['Tdark', 'Tbio', 'Tchem', 'Tclin']

NCHUNK=1000;

#############################################################################
def Info(dbcon, fout=None):
  sql = 'SELECT * FROM dbinfo'
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def ListTables(dbcon, fout=None):
  sql = 'SHOW TABLES'
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_table: {df.shape[0]}")
  return df

#############################################################################
def ListDatasets(dbcon, fout=None):
  sql = 'SELECT * FROM dataset'
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_table: {df.shape[0]}")
  return df

#############################################################################
def DescribeTables(dbcon, fout=None):
  return ListColumns(dbcon, fout)

#############################################################################
def ListColumns(dbcon, fout=None):
  n_table=0; df=pd.DataFrame();
  for table in ListTables(dbcon).iloc[:,0]:
    n_table+=1
    sql = ('DESCRIBE '+table)
    df_this = pd.read_sql(sql, dbcon)
    cols = list(df_this.columns.values)
    df_this['table'] = table
    df_this = df_this[['table']+cols]
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_columns: {df.shape[0]}; n_tables: {df.table.nunique()}")
  return df

#############################################################################
def TableRowCounts(dbcon, fout=None):
  n_table=0; df=pd.DataFrame();
  for table in ListTables(dbcon).iloc[:,0]:
    n_table+=1
    sql = ('SELECT count(*) AS row_count FROM '+table)
    df_this = pd.read_sql(sql, dbcon)
    cols = list(df_this.columns.values)
    df_this['table'] = table
    df_this = df_this[['table']+cols]
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_table: {df.shape[0]}")
  return df

#############################################################################
def TDLCounts(dbcon, fout=None):
  sql="SELECT tdl, COUNT(id) AS target_count FROM target GROUP BY tdl"
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def AttributeCounts(dbcon, fout=None):
  sql='SELECT ga.type, COUNT(ga.id) FROM gene_attribute ga GROUP BY ga.type'
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def XrefCounts(dbcon, fout=None):
  sql = ('SELECT xtype, COUNT(DISTINCT value) FROM xref GROUP BY xtype ORDER BY xtype')
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def ListXrefTypes(dbcon, fout=None):
  sql='SELECT DISTINCT xtype FROM xref ORDER BY xtype'
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_xreftypes: {df.shape[0]}")
  return df

#############################################################################
def ListXrefs(dbcon, xreftypes, fout=None):
  if xreftypes and type(xreftypes) not in (list, tuple):
    xreftypes = [xreftypes]
  cols=['target_id', 'protein_id', 'xtype', 'value'] 
  sql = f"""SELECT DISTINCT {(", ".join(cols))} FROM xref"""
  if xreftypes:
    sql += f""" WHERE xtype IN ({("'"+("','".join(xreftypes)+"'"))})"""
  logging.debug(f'SQL: "{sql}"')
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"Target count: {df.target_id.nunique()}")
  logging.info(f"Protein count: {df.protein_id.nunique()}")
  logging.info(f"XrefType count: {df.xtype.nunique()}")
  logging.info(f"Xref count: {df.value.nunique()}")
  return df

#############################################################################
def ListTargetFamilies(dbcon, fout=None):
  sql="SELECT DISTINCT fam TargetFamily FROM target ORDER BY fam"
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_tfams: {df.shape[0]}")
  return df

#############################################################################
def ListTargets(dbcon, tdls, tfams, fout=None):
  sql = """
SELECT
	target.id tcrdTargetId,
	target.name tcrdTargetName,
	target.fam tcrdTargetFamily,
	target.tdl TDL,
	target.ttype tcrdTargetType,
	target.idg idgList,
	protein.id tcrdProteinId,
	protein.uniprot uniprotId,
	protein.sym tcrdGeneSymbol,
	protein.family tcrdProteinFamily,
	protein.geneid ncbiGeneId,
	protein.up_version uniprotVersion,
	protein.chr,
	protein.description tcrdProteinDescription,
	protein.dtoid dtoId,
	protein.dtoclass dtoClass,
	protein.stringid ensemblProteinId,
	x.value ensemblGeneId
FROM
	target
LEFT OUTER JOIN
	t2tc ON t2tc.target_id = target.id
LEFT OUTER JOIN
	protein ON protein.id = t2tc.protein_id
LEFT OUTER JOIN
	(SELECT * FROM xref WHERE xref.xtype = 'Ensembl' AND xref.value REGEXP '^ENSG'
	) x ON x.protein_id = protein.id
"""
  wheres=[]
  if tdls: wheres.append(f"""target.tdl IN ({("'"+("','".join(tdls))+"'")})""")
  if tfams: wheres.append(f"""target.fam IN ({("'"+("','".join(tfams))+"'")})""")
  if wheres: sql+=(' WHERE '+(' AND '.join(wheres)))
  logging.debug(sql)
  df = pd.read_sql(sql, dbcon)
  if tdls: logging.info(f"""TDLs: {",".join(tdls)}""")
  if tfams: logging.info(f"""TargetFamilies: {",".join(tfams)}""")
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  logging.info(f"Target count: {df.tcrdTargetId.nunique()}")
  logging.info(f"Protein count: {df.tcrdProteinId.nunique()}")
  logging.info(f"Uniprot count: {df.uniprotId.nunique()}")
  logging.info(f"ENSP count: {df.ensemblProteinId.nunique()}")
  logging.info(f"ENSG count: {df.ensemblGeneId.nunique()}")
  logging.info(f"GeneSymbol count: {df.tcrdGeneSymbol.nunique()}")
  return df

#############################################################################
def ListTargetsByDTO(dbcon, fout=None):
  sql = """
SELECT DISTINCT
	target.id tcrdTargetId,
	target.name tcrdTargetName,
	target.fam tcrdTargetFamily,
	protein.id tcrdProteinId,
	protein.sym tcrdGeneSymbol,
	CONVERT(protein.geneid, CHAR) ncbiGeneId,
	protein.uniprot uniprotId,
	p2dto.dtoid dtoId,
	CONVERT(p2dto.generation, CHAR) dtoGeneration,
	dto.name dtoClass
FROM
	target
	JOIN t2tc ON t2tc.target_id = target.id
	JOIN protein ON protein.id = t2tc.protein_id
	LEFT OUTER JOIN p2dto ON p2dto.protein_id = protein.id
	LEFT OUTER JOIN dto ON dto.dtoid = p2dto.dtoid
ORDER BY
	target.id, protein.id, p2dto.generation DESC
"""
  logging.debug(sql)
  df = pd.read_sql(sql, dbcon, coerce_float=False)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  logging.info(f"Target count: {df.tcrdTargetId.nunique()}")
  logging.info(f"Protein count: {df.tcrdProteinId.nunique()}")
  logging.info(f"Uniprot count: {df.uniprotId.nunique()}")
  logging.info(f"GeneSymbol count: {df.tcrdGeneSymbol.nunique()}")
  logging.info(f"DTO_ID count: {df.dtoId.nunique()}")
  return df

#############################################################################
# E.g., TID 228 has 2 ensemblGeneIds.
def GetTargetPage(dbcon, tid, fout=None):
  df = GetTargets(dbcon, [tid], "TID", None)
  if df['ensemblGeneId'].nunique()>1:
    df.sort_values("ensemblGeneId", inplace=True)
    logging.warning(f"TID:{tid}; multiple ({df.shape[0]}) ensemblGeneIds: {','.join(df['ensemblGeneId'].tolist())} (keeping 1st only)")
    df = df.iloc[[0]]
  target = df.to_dict(orient='records')
  fout.write(json.dumps(target, indent=2))

#############################################################################
def GetTargets(dbcon, ids, idtype, fout=None):
  cols=[
	'target.id',
	'target.name',
	'target.description',
	'target.fam',
	'target.tdl',
	'target.ttype',
	'protein.id',
	'protein.sym',
	'protein.uniprot',
	'protein.geneid',
	'protein.name',
	'protein.description',
	'protein.family',
	'protein.chr']
  sql = f"""\
SELECT
	{(','.join(map(lambda s: (s+" "+s.replace('.', '_')), cols)))},
	x.value ensemblGeneId
FROM
	target
LEFT OUTER JOIN
	t2tc ON target.id = t2tc.target_id
LEFT OUTER JOIN
	protein ON protein.id = t2tc.protein_id
LEFT OUTER JOIN
	(SELECT * FROM xref WHERE xref.xtype = 'Ensembl' AND xref.value REGEXP '^ENSG'
	) x ON x.protein_id = protein.id
"""
  n_id=0; n_hit=0; df=pd.DataFrame();
  for id_this in ids:
    n_id+=1
    wheres=[]
    if idtype.lower()=='tid':
      wheres.append(f"target.id = {id_this}")
    elif idtype.lower()=='uniprot':
      wheres.append(f"protein.uniprot = '{id_this}'")
    elif idtype.lower() in ('gid', 'geneid'):
      wheres.append(f"protein.geneid = '{id_this}'")
    elif idtype.lower() in ('genesymb', 'genesymbol'):
      wheres.append(f"protein.sym = '{id_this}'")
    elif idtype.lower() == 'ensp':
      wheres.append(f"protein.stringid = '{id_this}'")
    else:
      logging.error(f'Unknown ID type: {idtype}')
      return
    sql_this = sql+(' WHERE '+(' AND '.join(wheres)))
    logging.debug(f'SQL: "{sql_this}"')
    logging.debug(f'ID: {idtype} = "{id_this}"')
    df_this = pd.read_sql(sql_this, dbcon)
    if df_this.shape[0]>0: n_hit+=1
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_id: {n_id}; n_hit: {n_hit}")
  return df

#############################################################################
def GetTargetsByXref(dbcon, ids, xreftypes, fout=None):
  if xreftypes and type(xreftypes) not in (list, tuple):
    xreftypes = [xreftypes]
  cols=[ 'target.description',
	'target.id',
	'target.fam',
	'target.name',
	'target.tdl',
	'target.ttype',
	'protein.chr',
	'protein.description',
	'protein.family',
	'protein.geneid',
	'protein.id',
	'protein.name',
	'protein.sym',
	'protein.uniprot',
	'xref.xtype',
	'xref.value']
  sql = f"""\
SELECT
	{(','.join(map(lambda s: s+" "+s.replace('.', '_')), cols))}
FROM
	target
JOIN
	t2tc ON target.id = t2tc.target_id
JOIN
	protein ON protein.id = t2tc.protein_id
JOIN
	xref ON xref.protein_id = protein.id
""" 

  n_id=0; n_hit=0; df=pd.DataFrame();
  for id_this in ids:
    n_id+=1
    wheres=[]
    wheres.append(f"""xref.xtype IN ({("'"+("','".join(xreftypes))+"'")})""")
    wheres.append(f"xref.value = '{id_this}'")
    sql_this = sql+('WHERE '+(' AND '.join(wheres)))
    logging.debug(f'SQL: "{sql_this}"')
    df_this = pd.read_sql(sql_this, dbcon)
    if df_this.shape[0]>0:
      n_hit+=1
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_id: {n_id}; n_hit: {n_hit}")
  return df

#############################################################################
def GetPathways(dbcon, ids, fout=None):
  n_id=0; n_hit=0; df=pd.DataFrame(); pids_all=set();
  cols=[ 't2p.target_id',
	't2p.id',
	't2p.source',
	't2p.id_in_source',
	't2p.name',
	't2p.description',
	't2p.url' ]
  sql = f"""SELECT {(','.join(map(lambda s: s+" "+s.replace('.', '_')), cols))} FROM target2pathway t2p JOIN target t ON t.id = t2p.target_id"""
  for id_this in ids:
    n_id+=1
    sql_this = sql+(f"WHERE t.id = {id_this}")
    logging.debug(f'SQL: "{sql_this}"')
    df_this = pd.read_sql(sql_this, dbcon)
    if df_this.shape[0]>0: n_hit+=1
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_query: {n_query}; n_hit: {n_hit}")
  return df

#############################################################################
def ListDiseases(dbcon, fout=None):
  "Ontologies: DOID, UMLS, MESH, AmyCo, OrphaNet, NCBIGene"
  sql="""
SELECT
	disease.name diseaseName,
	disease.did diseaseId,
	IF((disease.did REGEXP '^C[0-9]+$'), 'UMLS', SUBSTR(disease.did, 1, LOCATE(':', disease.did)-1)) diseaseOntology,
	disease.description diseaseDescription,
	disease.dtype associationType,
	disease_type.description associationDescription,
	disease.source associationSource,
	COUNT(disease.protein_id) nTargetAssociations
FROM
	disease
	LEFT OUTER JOIN disease_type ON disease_type.name = disease.dtype
GROUP BY
	disease.dtype,
	disease_type.description,
	disease.name,
	disease.did,
	disease.description,
	disease.source
"""
  df = pd.read_sql(sql, dbcon)
  if fout is not None: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  logging.info(f"diseaseIDs: {df.diseaseId.nunique()}")
  logging.info(f"diseaseNames: {df.diseaseName.nunique()}")
  for ontology in sorted(df.diseaseOntology.unique().astype(str).tolist()):
    if df[df.diseaseOntology==ontology].diseaseId.nunique()>0:
      logging.info(f"[{ontology}] diseaseIDs: {df[df.diseaseOntology==ontology].diseaseId.nunique()}")
  return df

#############################################################################
def ListDiseaseTypes(dbcon, fout=None):
  df = ListDiseases(dbcon, None)
  df = df[["associationType", "associationDescription", "nTargetAssociations"]]
  df = df.groupby(["associationType", "associationDescription"]).sum()
  df.reset_index(drop=False, inplace=True)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  return df

#############################################################################
def GetDiseaseAssociations(dbcon, ids, fout=None):
  n_id=0; n_hit=0; df=pd.DataFrame();
  sql = """\
SELECT
	disease.name diseaseName,
	disease.did diseaseId,
	IF((disease.did REGEXP '^C[0-9]+$'), 'UMLS', SUBSTR(disease.did, 1, LOCATE(':', disease.did)-1)) diseaseOntology,
	disease.description diseaseDescription,
	disease.dtype associationType,
	disease_type.description associationDescription,
	disease.source associationSource,
	protein.id tcrdProteinId,
	protein.uniprot uniprotId,
	protein.sym tcrdGeneSymbol
FROM
	disease
	LEFT OUTER JOIN disease_type ON disease_type.name = disease.dtype
	JOIN protein ON protein.id = disease.protein_id
WHERE
	disease.did = '{0}'
"""
  for id_this in ids:
    n_id+=1
    sql_this = sql.format(id_this)
    df_this = pd.read_sql(sql_this, dbcon)
    if df_this.shape[0]>0: n_hit+=1
    if fout is not None: df_this.to_csv(fout, "\t", index=False)
    else: df = pd.concat([df, df_this])
  logging.info(f"n_id: {n_id}; n_hit: {n_hit}")
  if fout is None: return df

#############################################################################
def GetDiseaseAssociationsPage(dbcon, did, fout=None):
  df = GetDiseaseAssociations(dbcon, [did], None)
  disease = df[["diseaseName", "diseaseId", "diseaseOntology", "diseaseDescription"]].drop_duplicates().to_dict(orient='records')[0]
  associations = df[["associationType", "associationDescription", "associationSource", "tcrdProteinId", "uniprotId", "tcrdGeneSymbol"]].to_dict(orient='records')
  disease["associations"] = associations
  fout.write(json.dumps(disease, indent=2))

#############################################################################
def ListPhenotypes(dbcon, fout=None):
  sql="""
SELECT
	p.ptype,
        CONCAT(IF(INSTR(IFNULL(p.trait, ""), ';')>0, SUBSTR(IFNULL(p.trait, ""), 1, INSTR(IFNULL(p.trait, ""), ';')-1), IFNULL(p.trait, "")),IFNULL(p.term_id, "")) p_identifier,
        p.term_name,
        p.term_description,
	pt.ontology ptype_ontology,
	pt.description ptype_description,
	COUNT(p.protein_id) n_target_associations
FROM
	phenotype p
	JOIN phenotype_type pt ON pt.name = p.ptype
GROUP BY
	p.ptype,
        p_identifier,
        p.term_name,
        p.term_description,
	pt.ontology,
	pt.description
"""
  df = pd.read_sql(sql, dbcon)
  df.loc[((df['ptype']=='OMIM') & ((df['ptype_ontology']=='')|(df['ptype_ontology'].isna()))),'ptype_ontology'] = 'OMIM' # Kludge
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  for ptype in df.ptype.unique().tolist():
    logging.info(f"[{ptype}] phenotype_IDs: {df[df.ptype==ptype].p_identifier.nunique()}")
  return df

#############################################################################
def ListPhenotypeTypes(dbcon, fout=None):
  df = ListPhenotypes(dbcon, None)
  df = df[["ptype", "ptype_ontology", "n_target_associations"]]
  df = df.groupby(["ptype", "ptype_ontology"]).sum()
  df.reset_index(drop=False, inplace=True)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"rows: {df.shape[0]}")
  return df

#############################################################################
def ListPublications(dbcon, fout=None):
  df=None; n_out=0; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  N_row = pd.read_sql("SELECT COUNT(*) FROM pubmed", dbcon).iloc[0,0]
  sql="""
SELECT
	id AS pmid,
	title,
	journal,
	date,
	authors,
	REPLACE(abstract, '\n', ' ') AS abstract
FROM
	pubmed
"""
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if not quiet and tq is None: tq = tqdm.tqdm(total=N_row)
    if fout is not None: df_this.to_csv(fout, "\t", header=bool(n_out==0), index=False)
    else: df = pd.concat([df, df_this])
    if tq is not None: tq.update(df_this.shape[0])
    n_out += df_this.shape[0]
  if tq is not None: tq.close()
  logging.info(f"rows: {n_out}")
  return df

#############################################################################
def ListCompounds(dbcon, fout=None):
  df=None; n_out=0; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  N_row = pd.read_sql("SELECT COUNT(DISTINCT cmpd_pubchem_cid,smiles) FROM cmpd_activity WHERE cmpd_pubchem_cid IS NOT NULL", dbcon).iloc[0,0]
  sql="""\
SELECT DISTINCT
	cmpd_pubchem_cid,
	smiles,
	COUNT(DISTINCT target_id) target_count,
	COUNT(DISTINCT id) activity_count
FROM cmpd_activity
WHERE cmpd_pubchem_cid IS NOT NULL
GROUP BY
	cmpd_pubchem_cid,smiles
"""
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if not quiet and tq is None: tq = tqdm.tqdm(total=N_row)
    if fout is not None: df_this.to_csv(fout, "\t", header=bool(n_out==0), index=False)
    else: df = pd.concat([df, df_this])
    if tq is not None: tq.update(df_this.shape[0])
    n_out += df_this.shape[0]
  if tq is not None: tq.close()
  logging.info(f"rows: {n_out}")
  return df

#############################################################################
def ListDrugs(dbcon, fout=None):
  df=None; n_out=0; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  N_row = pd.read_sql("SELECT COUNT(DISTINCT cmpd_pubchem_cid,dcid,smiles,drug) FROM drug_activity WHERE cmpd_pubchem_cid IS NOT NULL", dbcon).iloc[0,0]
  sql="""\
SELECT DISTINCT
	cmpd_pubchem_cid,
	dcid,
	smiles,
	drug,
	COUNT(DISTINCT target_id) target_count,
	COUNT(DISTINCT id) activity_count
FROM drug_activity
WHERE cmpd_pubchem_cid IS NOT NULL
GROUP BY
	cmpd_pubchem_cid,dcid,smiles,drug
"""
  logging.debug(sql)
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if not quiet and tq is None: tq = tqdm.tqdm(total=N_row)
    if fout is not None: df_this.to_csv(fout, "\t", header=bool(n_out==0), index=False)
    else: df = pd.concat([df, df_this])
    if tq is not None: tq.update(df_this.shape[0])
    n_out += df_this.shape[0]
  if tq is not None: tq.close()
  logging.info(f"rows: {n_out}")
  return df

#############################################################################



#!/usr/bin/env python3
"""
utility app for the AMP T2D REST API.
http://www.type2diabetesgenetics.org/
http://www.kp4cd.org/apis/t2d
http://52.54.103.84/kpn-kb-openapi/

DEPICT software (Pers, TH, et al., 2015)
"""
###
import sys,os,re,json,argparse,time,logging
#
from ... import amp
#
API_HOST='public.type2diabeteskb.org'
API_BASE_PATH='/dccservices'
#
#############################################################################
if __name__=='__main__':
  ops = ["list_tissues", "list_phenotypes", "depict_genepathway"]
  parser = argparse.ArgumentParser(description="AMP T2D REST client")
  parser.add_argument("op",choices=ops,help='operation')
  parser.add_argument("--i", dest="ifile", help="input IDs file")
  parser.add_argument("--ids", help="input IDs, comma-separated")
  parser.add_argument("--gene", help="query gene (e.g. SLC30A8)")
  parser.add_argument("--phenotype", default="T2D")
  parser.add_argument("--max_pval", type=float, default=.0005)
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("--skip", type=int, default=0)
  parser.add_argument("--nmax", type=int, default=0)
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL = 'http://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  if args.ifile:
    fin = open(args.ifile)
    ids=[]
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
    logging.info('input IDs: %d'%(len(ids)))
    fin.close()
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())

  t0=time.time()

  if args.op == 'list_tissues':
    amp.t2d.ListTissues(BASE_URL, fout)

  elif args.op == 'list_phenotypes':
    amp.t2d.ListPhenotypes(BASE_URL, fout)

  elif args.op == 'depict_genepathway':
    amp.t2d.DepictGenePathway(BASE_URL, args.gene, args.phenotype, args.max_pval, fout)

  else:
    parser.error('Invalid operation: %s'%args.op)

  logging.info('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))




#!/usr/bin/env python3
"""
PubMed iCite  REST API client
https://icite.od.nih.gov/api
"""
###
import sys,os,re,argparse,logging
#
from .. import icite

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='PubMed iCite REST API client utility', epilog='Publication metadata.')
  ops = ['get_stats']
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--ids", help="PubMed IDs, comma-separated (ex:25533513)")
  parser.add_argument("--i", dest="ifile", help="input file, PubMed IDs")
  parser.add_argument("--nmax", help="list: max to return")
  parser.add_argument("--year", help="list: year of publication")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--api_host", default=icite.API_HOST)
  parser.add_argument("--api_base_path", default=icite.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  parser.add_argument("-q", "--quiet", action="store_true", help="Suppress progress notification.")
  args = parser.parse_args()

  # logging.PROGRESS = 15 (custom)
  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>0 else logging.ERROR if args.quiet else 15))

  base_url='https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w", encoding="utf-8") if args.ofile else sys.stdout

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    logging.info('Input IDs: %d'%(len(ids)))
    fin.close()
  elif args.ids:
    ids = re.split(r'[\s,]+', args.ids.strip())

  if args.op == 'get_stats':
    if not ids: parser.error(f'Operation requires PMID[s]: {args.op}')
    icite.GetStats(ids, base_url, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

